def avprin(self, key="", effnu="", **kwargs):
    """Specifies how principal and vector sums are to be calculated.

    APDL Command: AVPRIN

    Parameters
    ----------
    key
        Averaging key:

        0 - Average the component values from the elements at a common node, then calculate
            the principal or vector sum from the averaged components
            (default).

        1 - Calculate the principal or vector sum values on a per element basis, then
            average these values from the elements at a common node.

    effnu
        Effective Poisson's ratio used for computing the von Mises
        equivalent strain (EQV). This command option is intended for use
        with line elements or in load case operations (LCOPER) only; ANSYS
        automatically selects the most appropriate effective Poisson's
        ratio, as discussed below.

    Notes
    -----
    Selects the method of combining components for certain derived nodal
    results when two or more elements connect to a common node.  The
    methods apply to the calculations of derived nodal principal stresses,
    principal strains, and vector sums for selects, sorts, and output
    [NSEL, NSORT, PRNSOL, PLNSOL, etc.].

    This command also defines the effective Poisson's ratio (EFFNU) used
    for equivalent strain calculations.  If you use EFFNU, the default
    effective Poisson's ratios shown below will be overridden for all
    elements by the EFFNU value. To return to the default settings, issue
    the RESET command. The default value for EFFNU is:

    Poisson's ratio as defined on the MP commands for EPEL and EPTH

    0.5 for EPPL and EPCR

    0.5 if the referenced material is hyperelastic

    0.0 for line elements (includes beam, link, and pipe elements, as well
    as discrete elements), cyclic symmetry analysis, mode superposition
    analyses (with MSUPkey = YES on the MXPAND command), and load case
    operations (LCOPER).

    For the von Mises equivalent strain (EQV), it is always computed using
    the average of the equivalent strains from the elements at a common
    node irrespective of the value of the averaging KEY. If EFFNU is input,
    though, the calculation will be performed according to the KEY setting.

    For a random vibration (PSD) analysis, issuing either AVPRIN,0 or
    AVPRIN,1 calculates the principal stresses using the appropriate
    averaging method. They are then used to determine SEQV.  The output
    will have non-zero values for the principal stresses.

    If AVPRIN is not issued, the Segalman-Fulcher method is used to
    calculate SEQV. This method does not calculate principal stresses, but
    directly calculates SEQV from the component stresses; therefore, the
    output will have zero values for the principal stresses. Beam and pipe
    elements are excluded

    This command is also valid in POST26, where applicable.

    See Combined Stresses and Strains in the Mechanical APDL Theory
    Reference for more information.
    """
    command = f"AVPRIN,{key},{effnu}"
    return self.run(command, **kwargs)


def avres(self, key="", opt="", **kwargs):
    """Specifies how results data will be averaged when PowerGraphics is

    APDL Command: AVRES
    enabled.

    Parameters
    ----------
    key
        Averaging key.

        1 - Average results at all common subgrid locations.

        2 - Average results at all common subgrid locations except where material type
            [MAT] discontinuities exist. This option is the default.

        3 - Average results at all common subgrid locations except where real constant
            [REAL] discontinuities exist.

        4 - Average results at all common subgrid locations except where material type
            [MAT] or real constant [REAL] discontinuities exist.

    opt
        Option to determine how results data are averaged.

        (blank) - Average surface results data using only the exterior element faces (default).

        FULL - Average surface results data using the exterior face and interior element data.

    Notes
    -----
    The AVRES command specifies how results data will be averaged at
    subgrid locations that are common to 2 or more elements.  The command
    is valid only when PowerGraphics is enabled (via the /GRAPHICS,POWER
    command).

    With PowerGraphics active (/GRAPHICS,POWER), the averaging scheme for
    surface data with interior element data included (AVRES,,FULL) and
    multiple facets per edge (/EFACET,2 or /EFACET,4) will yield differing
    minimum and maximum contour values depending on the  Z-Buffering
    options (/TYPE,,6 or /TYPE,,7).  When the Section data is not included
    in the averaging schemes (/TYPE,,7), the resulting absolute value for
    the midside node is significantly smaller.

    PowerGraphics does not average your stresses across discontinuous
    surfaces. The normals for various planes and facets are compared to a
    tolerance to determine continuity. The ANGLE value you specify in the
    /EDGE command is the tolerance for classifying surfaces as continuous
    or "coplanar."

    The command affects nodal solution contour plots (PLNSOL), nodal
    solution printout (PRNSOL), and subgrid solution results accessed
    through the Query Results function (under General Postprocessing) in
    the GUI.

    The command has no effect on the nodal degree of freedom solution
    values (UX, UY, UZ, TEMP, etc.).

    For cyclic symmetry mode-superposition harmonic solutions, AVRES,,FULL
    is not supported. Additionally, averaging does not occur across
    discontinuous surfaces, and the ANGLE value on the /EDGE command has no
    effect.

    The command is also available in /SOLU.
    """
    command = f"AVRES,{key},{opt}"
    return self.run(command, **kwargs)


def efacet(self, num="", **kwargs):
    """Specifies the number of facets per element edge for PowerGraphics

    APDL Command: /EFACET
    displays.

    .. warning::
       This will not change element plotting within PyMapdl with
       ``eplot(vtk=True)``

    Parameters
    ----------
    num
        Number of facets per element edge for element plots.

        1 - Use 1 facet per edge (default for h-elements).

        2 - Use 2 facets per edge.

        4 - Use 4 facets per edge.

    Notes
    -----
    /EFACET is valid only when PowerGraphics is enabled [/GRAPHICS,POWER],
    except that it can be used in FULL graphics mode for element CONTA174.
    (See the /GRAPHICS command and element CONTA174 in the Element
    Reference for more information.) The /EFACET command is only applicable
    to element type displays.

    /EFACET controls the fineness of the subgrid that is used for element
    plots. The element is subdivided into smaller portions called facets.
    Facets are piecewise linear surface approximations of the actual
    element face. In their most general form, facets are warped planes in
    3-D space.  A greater number of facets will result in a smoother
    representation of the element surface for element plots. /EFACET may
    affect results averaging. See Contour Displays in the Basic Analysis
    Guide for more information.

    For midside node elements, use NUM = 2; if NUM = 1, no midside node
    information is output. For non-midside node elements, NUM should be set
    to 1. See the PLNSOL and PRNSOL commands for more information.

    With PowerGraphics active (/GRAPHICS,POWER), the averaging scheme for
    surface data with interior element data included (AVRES,,FULL) and
    multiple facets per edge (/EFACET,2 or /EFACET,4) will yield differing
    minimum and maximum contour values depending on the  Z-Buffering
    options (/TYPE,,6 or /TYPE,,7).  When the Section data is not included
    in the averaging schemes (/TYPE,,7), the resulting absolute value for
    the midside node is significantly smaller.

    For cyclic symmetry mode-superposition harmonic solutions, only NUM = 1
    is supported in postprocessing.

    Caution:: : If you specify /EFACET,1, PowerGraphics does not plot
    midside nodes. You must use /EFACET,2 to make the nodes visible.

    This command is valid in any processor.
    """
    command = f"/EFACET,{num}"
    return self.run(command, **kwargs)


def ernorm(self, key="", **kwargs):
    """Controls error estimation calculations.

    APDL Command: ERNORM

    Parameters
    ----------
    key
        Control key:

        ON - Perform error estimation (default). This option is
        not valid for PowerGraphics.

        OFF - Do not perform error estimation.

    Notes
    -----
    Especially for thermal analyses, program speed increases if
    error estimation is suppressed.  Therefore, it might be
    desirable to use error estimation only when needed.  The value
    of the ERNORM key is not saved on file.db. Consequently, you
    need to reissue the ERNORM key after a RESUME if you wish to
    deactivate error estimation again.
    """
    return self.run("ERNORM,%s" % (str(key)), **kwargs)


def inres(self, item1="", item2="", item3="", item4="", item5="", item6="",
          item7="", item8="", **kwargs):
    """Identifies the data to be retrieved from the results file.

    APDL Command: INRES

    Parameters
    ----------
    item1, item2, item3, . . . , item8
        Data to be read into the database from the results file.  May
        consist of any of the following labels:

        ALL - All solution items (default).

        BASIC - NSOL, RSOL, NLOAD, STRS, FGRAD, and FFLUX items.

        NSOL - Nodal DOF solution.

        RSOL - Nodal reaction loads.

        ESOL - Element solution items (includes all of the following):

        NLOAD - Element nodal loads.

        STRS - Element nodal stresses.

        EPEL - Element elastic strains.

        EPTH - Element thermal, initial, and swelling strains.

        EPPL - Element plastic strains.

        EPCR - Element creep strains.

        FGRAD - Element nodal gradients.

        FFLUX - Element nodal fluxes.

        MISC - Element miscellaneous data (SMISC and NMISC).

    Notes
    -----
    Identifies the type of data to be retrieved from the results file for
    placement into the database through commands such as SET, SUBSET, and
    APPEND.  INRES is a companion command to the OUTRES command controlling
    data written to the database and the results file.  Since the INRES
    command can only flag data that has already been written to the results
    file, care should be taken when using the OUTRES command to include all
    data you wish to retrieve for postprocessing later on.
    """
    command = f"INRES,{item1},{item2},{item3},{item4},{item5},{item6},{item7},{item8}"
    return self.run(command, **kwargs)


def layer(self, num="", **kwargs):
    """Specifies the element layer for which data are to be processed.

    APDL Command: LAYER

    Parameters
    ----------
    num
        Layer-processing mode:

        N - The layer number to process. The default value is 0.

        FCMAX - Processes the layer with the largest failure criteria.

    Notes
    -----
    Specifies the element layer for which results data are to be listed,
    plotted, or otherwise processed.

    Applies to stress and strain data for layered elements SHELL163,
    SHELL181, SHELL281, ELBOW290, SOLID185, SOLID186, SOLSH190, SHELL208,
    SHELL209, REINF264, and REINF265; heat flux and heat gradient for
    SHELL131 and SHELL132.

    The SHELL command may then be used (with shell elements) to specify a
    location (TOP, MID, BOT) within the layer for output. (The SHELL
    command does not apply to thermal shell elements SHELL131 and
    SHELL132.) Transverse shear stresses for MID are linearly averaged from
    TOP and BOT, and do not reflect a parabolic distribution. Setting
    KEYOPT(8) = 2 for SHELL181, SHELL281, SHELL208, SHELL209, and ELBOW290
    writes the mid-surface values directly to the results file and yields
    more accurate values than linear averaging.

    Because energy is a per-element quantity, you cannot use this command
    for energy output.

    When using the LAYER command with SHELL181, SOLID185, SOLID186,
    SOLSH190, SHELL208, SHELL209, SHELL281, and ELBOW290, KEYOPT(8) must be
    set to 1 (or 2 for SHELL181, SHELL281, ELBOW290, SHELL208, and
    SHELL209) in order to store results for all layers.

    When NUM = FCMAX, you must provide the failure criterion input. If
    specifying input via the FC command, all structural elements are
    processed. For more information, see the documentation for the FC
    command.

    Use this command with RSYS,LSYS to display results in the layer
    coordinate system for a particular layer.

    For the ANSYS LS-DYNA product, this command works differently than
    described above.  For SHELL163, you must first use EDINT during the
    solution phase to define the integration points for which you want
    output data.  Be aware that the output location for SHELL163 data is
    always at the integration point, so "top" and "bottom" refer to the top
    or bottom integration point, not necessarily the top or bottom surface.
    For more information, see the ANSYS LS-DYNA User's Guide.
    """
    command = f"LAYER,{num}"
    return self.run(command, **kwargs)


def rsys(self, kcn="", **kwargs):
    """Activates a coordinate system for printout or display of element and

    APDL Command: RSYS
    nodal results.

    Parameters
    ----------
    kcn
        The coordinate system to use for results output:

        0 - Global Cartesian coordinate system (default, except for spectrum analyses).

        1 - Global cylindrical coordinate system.

        2 - Global spherical coordinate system.

        > 10 - Any existing local coordinate system.

        SOLU - Solution coordinate systems. For element quantities, these are the element
               coordinate system for each element.  For nodal quantities,
               these are the nodal coordinate systems. If an element or
               nodal coordinate system is not defined, ANSYS uses the
               global Cartesian coordinate system. If you issue a LAYER,N
               command (where N refers to a layer number), the results
               appear in the layer coordinate system. (SOLU is the default
               for spectrum analyses.)

        LSYS - Layer coordinate system. For layered shell and solid elements, the results
               appear in their respective layer coordinate systems.  For a
               specific layer of interest, issue a LAYER,N command (where N
               refers to a layer number). If a model has both nonlayered
               and layered elements, you can use RSYS,SOLU and RSYS,LSYS
               simultaneously (with RSYS,SOLU applicable to nonlayered
               elements and RSYS,LSYS applicable to layered elements).  To
               reverse effects of the LSYS option, issue an RSYS,0 command.
               LSYS is the default for spectrum analysis.

    Notes
    -----
    The RSYS command activates a coordinate system for printing or
    displaying element results data such as stresses and heat fluxes, and
    nodal results data such as degrees of freedom and reactions. ANSYS
    rotates the results data to the specified coordinate system during
    printout, display, or element table operations (such as PRNSOL, PRESOL,
    PLNSOL, and ETABLE). You can define coordinate systems with various
    ANSYS commands such as LOCAL, CS, CLOCAL, and CSKP.

    If you issue RSYS with KCN > 10 (indicating a local coordinate system),
    and the specified system is subsequently redefined, you must reissue
    RSYS for results to be rotated into the redefined system.

    Note:: : The default coordinate system for certain elements, notably
    shells, is not global Cartesian and is frequently not aligned at
    adjacent elements.

    The use of RSYS,SOLU with these elements can make nodal averaging of
    component element results, such as SX, SY, SZ, SXY, SYZ, and SXZ,
    invalid and is not recommended.

    The RSYS command has no effect on beam or pipe stresses, which ANSYS
    displays (via /ESHAPE,1 and PowerGraphics) in the element coordinate
    system.

    Element results such as stresses and heat fluxes are in the element
    coordinate systems when KCN = SOLU. Nodal requests for element results
    (for example, PRNSOL,S,COMP) average the element values at the common
    node; that is, the orientation of the node is not a factor in the
    output of element quantities.  For nearly all solid elements, the
    default element coordinate systems are parallel to the global Cartesian
    coordinate system. For shell elements and the remaining solid elements,
    the default element coordinate system can differ from element to
    element. For layered shell and layered solid elements, ANSYS initially
    selects the element coordinate system when KCN = SOLU; you can then
    select the layer coordinate system via the LAYER command.

    Nodal results such as degrees of freedom and reactions can be properly
    rotated only if the resulting component set is consistent with the
    degree-of-freedom set at the node. (The degree-of-freedom set at a node
    is determined by the elements attached to the node.) For example, if a
    node does not have a UZ degree of freedom during solution, then any Z
    component resulting from a rotation does not print or display in POST1.
    Therefore, results at nodes with a single degree-of-freedom (UY only,
    for example) should not be rotated; that is, they should be viewed only
    in the nodal coordinate system or a system parallel to the nodal
    system. (The global Cartesian system--the RSYS command default--may not
    be parallel to the nodal system.) Results at nodes with a 2-D degree-
    of-freedom set (UX and UY, for example) should not be rotated out of
    the 2-D plane.

    PowerGraphics

    For PowerGraphics, ANSYS plots PLVECT vector arrow displays (such
    temperature, velocity, and force) in the global Cartesian coordinate
    system (RSYS = 0). Subsequent operations revert to your original
    coordinate system.

    PGR File

    When you generate a .PGR file in SOLUTION, you can use the Results
    Viewer to display your stresses only in the coordinate system in which
    you write your .PGR file. To view stresses in other coordinate systems,
    load your results file into the Results Viewer and regenerate the data.

    Large Deflections

    If large deflection is active (NLGEOM,ON), ANSYS rotates the element
    component result directions by the amount of rigid body rotation.

    ANSYS displays the element component results in the initial global
    coordinate system for the following elements: SHELL181, SHELL281,
    ELBOW290, PLANE182, PLANE183, SOLID185, SOLID186, SOLID187, SOLID272,
    SOLID273, SOLID285, SOLSH190, SHELL208, and SHELL209. All other element
    result transformations are, therefore, also relative to the initial
    global system. Nodal degree-of-freedom results are based on the initial
    (and not the updated) geometry. For all other element types, component
    results displayed in the co-rotated coordinate system include the
    element rigid body rotation from the initial global coordinate system,
    and all other element result transformations are relative to the
    rotated global system.

    LS-DYNA

    You can use the RSYS command to rotate stress data for all explicit
    (ANSYS LS-DYNA) elements except BEAM161, COMBI165, and composite
    SHELL163 (KEYOPT(3) = 1). In models that contain these element types
    combined with other explicit elements, you must unselect the
    unsupported elements before issuing the RSYS command. The command does
    not support strain data for any explicit element types. If you request
    strain results for explicit elements when RSYS is not set to the global
    Cartesian coordinate system (KCN = 0), ANSYS ignores the printing or
    plotting command. (ANSYS always rotates displacements into the results
    coordinate system, independent of the explicit element type.)
    """
    command = f"RSYS,{kcn}"
    return self.run(command, **kwargs)



def ftype(self, filetype="", prestype="", **kwargs):
    """Specifies the file type and pressure type for the subsequent import of

    APDL Command: FTYPE
    source points and pressures.

    Parameters
    ----------
    filetype
        Type of file from which the pressure data will be retrieved (no
        default):

        CFXTBR - File from a CFX Transient Blade Row (TBR) analysis export.

        CFDPOST - File from a CFD-Post BC Profile export.

        FORMATTED - Formatted file.

        CSV - Comma-Separated Values file.

    prestype
        Type of pressure data contained in the file:

        0 - Only real-valued pressures are on the file.

        1 - Real-valued and imaginary-valued pressures are on the file (default).

    Notes
    -----
    CFX Transient Blade Row files (FileType = CFXTBR) are obtained from the
    Export Results Tab in CFX-Pre, with [Export Surface Name]: Option set
    to Harmonic Forced Response.

    CFD-Post files (FileType = CFDPOST) are obtained from the Export action
    in CFD-Post with Type set to BC Profile.

    Formatted files (FileType = FORMATTED) contain the coordinates and
    pressure data in fixed-format columns in the order  x, y, z, pressure.
    You may have other columns of data in the file which can be skipped
    over in the Format specifier on the READ command, but the data must be
    in that order.

    Comma-separated values files (FileType = CSV) contain the coordinates
    and pressure data in comma-separated fields. The data can be in any
    order, and other fields of data may also be present.
    """
    command = f"FTYPE,{filetype},{prestype}"
    return self.run(command, **kwargs)


def map_(self, kdim="", kout="", limit="", **kwargs):
    """Maps pressures from source points to target surface elements.

    APDL Command: MAP

    kdim
        Interpolation key:

        * ``"0 or 2"`` : Interpolation is done on a surface (default).

        * ``"3"`` : Interpolation is done within a volume. This option
          is useful if the supplied source data is volumetric field
          data rather than surface data.

    kout
        Key to control how pressure is applied when a target node is
        outside of the source region:

        * ``"0"`` : Use the pressure(s) of the nearest source point for target nodes outside of the region (default).

        * ``"1"`` : Set pressures outside of the region to zero.

    limit
        Number of nearby points considered for interpolation. The minimum
        is 5; the default is 20. Lower values reduce processing
        time. However, some distorted or irregular meshes will require a
        higher LIMIT value to find the points encompassing the target node
        in order to define the region for interpolation.

    Notes
    -----
    Maps pressures from source points to target surface elements.
    """
    return self.run(f"MAP,,{kdim},,{kout},{limit}", **kwargs)


def plgeom(self, item="", nodekey="", **kwargs):
    """Plots target and source geometries.

    APDL Command: PLGEOM

    Parameters
    ----------
    item
        Items to plot:

        BOTH - Plot both target and source geometries (default).

        TARGET - Plot only the target geometry.

        SOURCE - Plot only the source geometry.

    nodekey
        If the source data contains faces (that is, surface elements were
        created upon the READ command), set NODEkey = 1 to plot only the
        source nodes rather than both the nodes and the elements.

    Notes
    -----
    Target faces are displayed in gray and source points in yellow. If the
    source data contains faces (that is, surface elements were created upon
    the READ command), the source faces are also displayed in blue (unless
    NODEkey = 1), and both surfaces are made translucent.
    """
    command = f"PLGEOM,{item},{nodekey}"
    return self.run(command, **kwargs)


def plmap(self, item="", nodekey="", imagkey="", **kwargs):
    """Plots target and source pressures.

    APDL Command: PLMAP

    Parameters
    ----------
    item
        Items to plot:

        BOTH - Plot both target and source pressures (default).

        TARGET - Plot only the target pressures.

        SOURCE - Plot only the source pressures.

    nodekey
        If the source data contains faces (that is, surface
        elements were created upon the READ command), set NODEkey
        = 1 to plot only the source nodes rather than both the
        nodes and the elements.

    imagkey
        1 - Plot the real pressures (default).

        0 - Plot the imaginary pressures.

    Notes
    -----
    Pressures on the target faces are displayed as a color contour
    plot using the command /PSF,PRES,,3. If the source data
    contains faces (that is, surface elements were created upon
    the READ command), the source faces are also displayed using a
    color contour plot by default. If NODEkey = 1 or no source
    faces are available, the source pressures are displayed as
    colored node symbols (/PSYMB,DOT,1 command).
    """
    return self.run(f"PLMAP,{item},,{nodekey},{imagkey}", **kwargs)


def read(self, fname="", nskip="", format_="", xfield="", yfield="",
         zfield="", prfield="", pifield="", **kwargs):
    """Reads coordinate and pressure data from a file.

    APDL Command: READ

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    nskip
        Number of lines at the beginning of the file that will be skipped
        while it is read. Default = 0. NSKIP is ignored for FileType =
        CFXTBR or CFDPOST on the FTYPE command.

    format\_
        For FileType = FORMATTED on the FTYPE command, Format is the read
        format in the FORTRAN FORMAT convention enclosed in parentheses;
        for example: (3e10.0,10x,e10.0,70x,e10.0)

    xfield, yfield, zfield, prfield, pifield
        For FileType = CSV on the FTYPE command, these are field numbers
        locating the coordinates and real and imaginary (if present)
        pressures. The field value may not exceed 20.

    Notes
    -----
    Reads coordinate and pressure data from the specified file. The file
    type must have been previously specified on the FTYPE command.

    Upon reading the file, nodes are created for the source points. For
    FileType = CFXTBR or CFDPOST on the FTYPE command, if face data is
    available, SURF154 elements are also created. A nodal component named
    SOURCENODES and an element component named SOURCEELEMS are created
    automatically.
    """
    command = f"READ,{fname},{nskip},{format_},{xfield},{yfield},{zfield},{prfield},{pifield}"
    return self.run(command, **kwargs)


def slashmap(self, **kwargs):
    """Enters the mapping processor.

    APDL Command: /MAP

    Notes
    -----
    Enters the mapping processor. This processor is used to read in source
    data from an external file and map it to the existing geometry.

    The current database is saved (to BeforeMapping.DB) upon entering the
    processor, and it is resumed upon exiting (FINISH command). Any nodes
    or elements not on the target surface are deleted for easier viewing of
    the mapping quantities. A database of this mapping geometry
    (Mapping.DB) is also saved at the FINISH command.

    This command is valid only at the Begin Level.
    """
    command = f"/MAP,"
    return self.run(command, **kwargs)


def target(self, nlist="", **kwargs):
    """Specifies the target nodes for mapping pressures onto surface effect

    APDL Command: TARGET
    elements.

    Parameters
    ----------
    nlist
        Nodes defining the surface upon which the pressures will be mapped.
        Use the label ALL or specify a nodal component name. If ALL, all
        selected nodes [NSEL] are used (default). Individual nodes may not
        be entered.

    Notes
    -----
    The node list specified by Nlist must contain a sufficient number of
    nodes to define an element surface. The surface must be meshed (ESURF
    command) with SURF154 elements prior to issuing this command.
    """
    command = f"TARGET,{nlist}"
    return self.run(command, **kwargs)


def writemap(self, fname="", **kwargs):
    """Writes interpolated pressure data to a file.

    APDL Command: WRITEMAP

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    Notes
    -----
    Writes the interpolated pressure data to the specified
    file. The data is written as SFE commands applied to the
    SURF154 elements that are on the target surface. You may read
    this data for inclusion in an analysis by using /INPUT,Fname.
    """
    return self.run(f"WRITEMAP,{fname}", **kwargs)



def addam(self, af="", aa="", ab="", ac="", ad="", amin="", **kwargs):
    """Specifies the acceleration spectrum computation constants for the

    APDL Command: ADDAM
    analysis of shock resistance of shipboard structures.

    Parameters
    ----------
    af
        Direction-dependent acceleration coefficient for elastic or
        elastic-plastic analysis option (default = 0).

    aa, ab, ac, ad
        Coefficients for the DDAM acceleration spectrum equations. Default
        for these coefficients is zero.

    amin
        The minimum acceleration value in inch/sec2.  It defaults to 2316
        inch/sec2 which equals 6g, where g is acceleration due to gravity
        (g = 386 inch/sec2).

    Notes
    -----
    This command specifies acceleration coefficients to analyze shock
    resistance of shipboard equipment.  These coefficients are used to
    compute mode coefficients according to the equations given in Dynamic
    Design Analysis Method in the Mechanical APDL Theory Reference.  The
    form of these equations is based on the Naval NRL Dynamic Design
    Analysis Method.  This command, along with the VDDAM and SED commands,
    is used with the spectrum (ANTYPE,SPECTR) analysis as a special purpose
    alternative to the SV, FREQ, and SVTYP commands.  The mass and length
    units of the model must be in pounds and inches, respectively.

    DDASPEC may alternatively be used to calculate spectrum coefficients.

    This command is also valid in PREP7.
    """
    command = f"ADDAM,{af},{aa},{ab},{ac},{ad},{amin}"
    return self.run(command, **kwargs)


def coval(self, tblno1="", tblno2="", sv1="", sv2="", sv3="", sv4="",
          sv5="", sv6="", sv7="", **kwargs):
    """Defines PSD cospectral values.

    APDL Command: COVAL

    Parameters
    ----------
    tblno1
        First input PSD table number associated with this spectrum.

    tblno2
        Second input PSD table number associated with this spectrum.

    sv1, sv2, sv3, . . . , sv7
        PSD cospectral values corresponding to the frequency points
        [PSDFRQ].

    Notes
    -----
    Defines PSD cospectral values to be associated with the previously
    defined frequency points.  Two table references are required since
    values are off-diagonal terms.  Unlike autospectra [PSDVAL], the
    cospectra can be positive or negative.  The cospectral curve segment
    where there is a sign change is interpolated linearly (the rest of the
    curve segments use log-log interpolation).  For better accuracy, choose
    as small a curve segment as possible wherever a sign change occurs.

    Repeat COVAL command using the same table numbers for additional
    points.  This command is valid for SPOPT,PSD only.

    This command is also valid in PREP7.
    """
    command = f"COVAL,{tblno1},{tblno2},{sv1},{sv2},{sv3},{sv4},{sv5},{sv6},{sv7}"
    return self.run(command, **kwargs)


def cqc(self, signif="", label="", forcetype="", **kwargs):
    """Specifies the complete quadratic mode combination method.

    APDL Command: CQC

    Parameters
    ----------
    signif
        Combine only those modes whose significance level exceeds the
        SIGNIF threshold.  For single point, multipoint, or DDAM response
        (SPOPT,SPRS, MPRS or DDAM), the significance level of a mode is
        defined as the mode coefficient of the mode, divided by the maximum
        mode coefficient of all modes.  Any mode whose significance level
        is less than SIGNIF is considered insignificant and is not
        contributed to the mode combinations.  The higher the SIGNIF
        threshold, the fewer the number of modes combined.  SIGNIF defaults
        to 0.001.  If SIGNIF is specified as 0.0, it is taken as 0.0.
        (This mode combination method is not valid for SPOPT,PSD.)

    label
        Label identifying the combined mode solution output.

        DISP - Displacement solution (default).  Displacements, stresses, forces, etc., are
               available.

        VELO - Velocity solution.  Velocities, "stress velocities," "force velocities," etc.,
               are available.

        ACEL - Acceleration solution.  Accelerations, "stress accelerations," "force
               accelerations," etc., are available.

    forcetype
        Label identifying the forces to be combined:

        STATIC - Combine the modal static forces (default).

        TOTAL - Combine the modal static plus inertial forces.

    Notes
    -----
    Damping is required for this mode combination method.  The CQC command
    is also valid for PREP7.
    """
    command = f"CQC,{signif},{label},{forcetype}"
    return self.run(command, **kwargs)


def ddaspec(self, keyref="", shptyp="", mountloc="", deftyp="", amin="",
            **kwargs):
    """APDL Command: DDASPEC

    Specifies the shock spectrum computation constants for DDAM analysis.

    Parameters
    ----------
    keyref
        Key for reference catalog:

        1 - The spectrum computation constants are based on NRL-1396 (default). For more
            information, see Dynamic Design Analysis Method in the
            Mechanical APDL Theory Reference

    shptyp
        Select the ship type:

        SUBM - Submarine

        SURF - Surface ship

    mountloc
        Select the mounting location:

        HULL - Hull mounting location. These structures are mounted directly to basic hull
               structures like frames, structural bulkheads below the water
               line, and shell plating above the water line.

        DECK - Deck mounting location. These structures are mounted directly to decks, non-
               structural bulkheads, or to structural bulkheads above the
               water line.

        SHEL - Shell plating mounting location. These structures are mounted directly to shell
               plating below the water line without intervening
               foundations.

    deftyp
        Select the deformation type:

        ELAS - Elastic deformation (default)

        PLAS - Elastic-plastic deformation

    amin
        Minimum acceleration value in inch/sec2. It defaults to 2316
        inch/sec2 which equals 6g, where g is the acceleration due to
        gravity (g = 386 in/sec2).

    Notes
    -----
    The excitation direction is required to calculate the spectrum
    coefficients. Issue the SED command before issuing DDASPEC.

    ADDAM and VDDAM may alternatively be used to calculate spectrum
    coefficients.

    This command is also valid in PREP7.
    """
    command = f"DDASPEC,{keyref},{shptyp},{mountloc},{deftyp},{amin}"
    return self.run(command, **kwargs)


def dsum(self, signif="", label="", td="", forcetype="", **kwargs):
    """Specifies the double sum mode combination method.

    APDL Command: DSUM

    Parameters
    ----------
    signif
        Combine only those modes whose significance level exceeds the
        SIGNIF threshold.  For single point, multipoint, or DDAM response
        (SPOPT, SPRS, MPRS, or DDAM), the significance level of a mode is
        defined as the mode coefficient of the mode, divided by the maximum
        mode coefficient of all modes.  Any mode whose significance level
        is less than SIGNIF is considered insignificant and is not
        contributed to the mode combinations.  The higher the SIGNIF
        threshold, the fewer the number of modes combined. SIGNIF defaults
        to 0.001.  If SIGNIF is specified as 0.0, it is taken as 0.0.
        (This mode combination method is not valid for SPOPT, PSD.)

    label
        Label identifying the combined mode solution output.

        DISP - Displacement solution (default).  Displacements, stresses, forces, etc., are
               available.

        VELO - Velocity solution.  Velocities, "stress velocities," "force velocities," etc.,
               are available.

        ACEL - Acceleration solution.  Accelerations, "stress accelerations," "force
               accelerations," etc., are available.

    td
        Time duration for earthquake or shock spectrum.  TD defaults to 10.

    forcetype
        Label identifying the forces to be combined:

        STATIC - Combine the modal static forces (default).

        TOTAL - Combine the modal static plus inertial forces.

    Notes
    -----
    This command is also valid for PREP7.
    """
    command = f"DSUM,{signif},{label},{td},{forcetype}"
    return self.run(command, **kwargs)


def freq(self, freq1="", freq2="", freq3="", freq4="", freq5="", freq6="",
         freq7="", freq8="", freq9="", **kwargs):
    """Defines the frequency points for the SV vs. FREQ tables.

    APDL Command: FREQ

    Parameters
    ----------
    freq1, freq2, freq3, . . . , freq9
        Frequency points for SV vs. FREQ tables.  Values must be in
        ascending order. FREQ1 should be greater than zero.  Units are
        cycles/time.

    Notes
    -----
    Repeat the FREQ command for additional frequency points (100 maximum).
    Values are added after the last nonzero frequency.  If all fields
    (FREQ1 -- FREQ9) are blank, erase SV vs. FREQ tables.

    Frequencies must be in ascending order.

    Spectral values are input with the SV command and interpreted according
    to the SVTYP command.  Applies only to the SPRS (single-point) option
    of the SPOPT command.  See the SPFREQ command for frequency input in
    MPRS (multi-point) analysis.

    Use the STAT command to list current frequency points.

    This command is also valid in PREP7.
    """
    command = f"FREQ,{freq1},{freq2},{freq3},{freq4},{freq5},{freq6},{freq7},{freq8},{freq9}"
    return self.run(command, **kwargs)


def grp(self, signif="", label="", forcetype="", **kwargs):
    """Specifies the grouping mode combination method.

    APDL Command: GRP

    Parameters
    ----------
    signif
        Combine only those modes whose significance level exceeds the
        SIGNIF  threshold.  For single point, multipoint, or DDAM response
        (SPOPT,SPRS, MPRS or DDAM), the significance level of a mode is
        defined as the mode coefficient of the mode, divided by the maximum
        mode coefficient of all modes.  Any mode whose significance level
        is less than SIGNIF is considered insignificant and is not
        contributed to the mode combinations.  The higher the SIGNIF
        threshold, the fewer the number of modes combined.  SIGNIF defaults
        to 0.001.  If SIGNIF is specified as 0.0, it is taken as 0.0.
        (This mode combination method is not valid for SPOPT,PSD.)

    label
        Label identifying the combined mode solution output.

        DISP - Displacement solution (default).  Displacements, stresses, forces, etc., are
               available.

        VELO - Velocity solution.  Velocities, "stress velocities," "force velocities," etc.,
               are available.

        ACEL - Acceleration solution.  Accelerations, "stress accelerations," "force
               accelerations," etc., are available.

    forcetype
        Label identifying the forces to be combined:

        STATIC - Combine the modal static forces (default).

        TOTAL - Combine the modal static plus inertial forces.

    Notes
    -----
    The SIGNIF value set with this command (including the default value of
    0.001) overrides the SIGNIF value set with the MXPAND command.

    This command is also valid for PREP7.
    """
    command = f"GRP,{signif},{label},{forcetype}"
    return self.run(command, **kwargs)


def mmass(self, option="", zpa="", **kwargs):
    """Specifies the missing mass response calculation.

    APDL Command: MMASS

    Parameters
    ----------
    option
        Flag to activate or deactivate missing mass response calculation.

        0 (OFF or NO) - Deactivate (default).

        1 (ON or YES) - Activate.

    zpa
        Zero Period Acceleration Value. If a scale factor FACT is defined
        on the SVTYP command, it is applied to this value.

    Notes
    -----
    The missing mass calculation is valid only for single point excitation
    response spectrum analysis (SPOPT, SPRS) and for multiple point
    response spectrum analysis (SPOPT, MPRS) performed with base excitation
    using acceleration response spectrum loading. Missing mass is supported
    in a spectrum analysis only when the preceding modal analysis is
    performed with the Block Lanczos, PCG Lanczos, Supernode, or Subspace
    eigensolver (Method =LANB, LANPCG, SNODE, or SUBSP on the MODOPT
    command).

    The velocity solution is not available (Label = VELO on the combination
    command: SRSS, CQC...) when the missing mass calculation is activated.

    The missing mass calculation is not supported when the spectrum
    analysis is based on a linear perturbation modal analysis performed
    after a nonlinear base analysis.

    The missing mass is not supported when superelements are present.

    To take into account the contribution of the truncated modes, the
    residual vector (RESVEC) can be used in place of the missing mass
    response. This is of particular interest if the velocity solution is
    requested or if a nonlinear prestress is included in the analysis
    (linear perturbation), or if a superelement is present, since the
    missing mass cannot be used in these cases.

    In a multiple point response spectrum analysis (SPOPT,MPRS), the MMASS
    command must precede the participation factor calculation command
    (PFACT).

    This command is also valid in PREP7.
    """
    command = f"MMASS,{option},{zpa}"
    return self.run(command, **kwargs)


def nrlsum(self, signif="", label="", labelcsm="", forcetype="", **kwargs):
    """Specifies the Naval Research Laboratory (NRL) sum mode combination

    APDL Command: NRLSUM
    method.

    Parameters
    ----------
    signif
        Combine only those modes whose significance level exceeds the
        SIGNIF threshold. For single point, multipoint, or DDAM response
        (SPOPT,SPRS, MPRS or DDAM), the significance level of a mode is
        defined as the mode coefficient of the mode, divided by the maximum
        mode coefficient of all modes. Any mode whose significance level is
        less than SIGNIF is considered insignificant and is not contributed
        to the mode combinations. The higher the SIGNIF threshold, the
        fewer the number of modes combined. SIGNIF defaults to 0.001. If
        SIGNIF is specified as 0.0, it is taken as 0.0. (This mode
        combination method is not valid for SPOPT,PSD.)

    label
        Label identifying the combined mode solution output.

        DISP - Displacement solution (default). Displacements, stresses, forces, etc., are
               available.

        VELO - Velocity solution. Velocities, "stress velocities," "force velocities," etc.,
               are available.

        ACEL - Acceleration solution. Accelerations, "stress accelerations," "force
               accelerations," etc., are available.

    labelcsm
        Label identifying the CSM (Closely Spaced Modes) method.

        CSM - Use the CSM method.

        Blank  - Do not use the CSM method (default).

    forcetype
        Label identifying the forces to be combined:

        STATIC - Combine the modal static forces (default).

        TOTAL - Combine the modal static plus inertial forces.

    Notes
    -----
    This command is also valid in PREP7. This mode combination method is
    usually used for SPOPT,DDAM.

    This CSM method is only applicable in a DDAM analysis (SPOPT,DDAM). The
    CSM method combines two closely spaced modes into one mode when their
    frequencies are within 10 percent of the common mean frequency and
    their responses are opposite in sign. The contribution of these closely
    spaced modes is then included in the NRL sum as a single effective
    mode. Refer to Closely Spaced Modes (CSM) Method in the Mechanical APDL
    Theory Reference for more information.

    NRLSUM is not allowed in ANSYS Professional.
    """
    command = f"NRLSUM,{signif},{label},{labelcsm},{forcetype}"
    return self.run(command, **kwargs)


def pfact(self, tblno="", excit="", parcor="", **kwargs):
    """Calculates participation factors for the PSD or multi-point response

    APDL Command: PFACT
    spectrum table.

    Parameters
    ----------
    tblno
        Input PSD (Power Spectral Density) table number for which
        participation factors are to be calculated.

    excit
        Label defining the location of excitation:

        BASE - Base excitation (default).

        NODE - Nodal excitation.

    parcor
        Label defining excitation type (applies only to SPOPT,PSD
        analysis).  Used only when partially correlated excitation is due
        to wave propagation or spatial correlation.  Defaults to partially
        correlated excitation as defined by COVAL and QDVAL commands.

        WAVE - Excitation defined by PSDWAV command.

        SPAT - Excitation defined by PSDSPL command.

    Notes
    -----
    Calculates the participation factors for a particular PSD or multi-
    point response spectrum table defined with the PSDVAL or SPVAL command.
    The Jobname.DB file must contain modal solution data in order for this
    command to calculate the participation factor.  There must be a PFACT
    command for each excitation spectrum. You are limited to 300
    excitations.

    This command is also valid in PREP7.
    """
    command = f"PFACT,{tblno},{excit},{parcor}"
    return self.run(command, **kwargs)


def pivcheck(self, key="", prntcntrl="", **kwargs):
    """Controls the behavior of an analysis when a negative or zero equation solver pivot value is encountered.

    APDL Command: PIVCHECK

    Parameters
    ----------
    key
        Determines whether to stop or continue an analysis when a negative
        or zero equation solver pivot value is encountered:

        AUTO - Check for negative or zero pivot values for analyses
               performed with the sparse and PCG solvers. When one is
               encountered, an error or warning is issued, per various
               criteria relating to the type of analysis being
               solved. An error causes the analysis to stop; a warning
               allows the analysis to continue. A negative pivot value
               may be valid for some nonlinear and multiphysics
               analyses (for example, electromagnetic and thermal
               analyses); this key has no effect in these cases.

        ERROR - Check for negative or zero pivot values for analyses
                performed with the sparse and PCG solvers. When one is
                encountered, an error is issued, stopping the
                analysis. A negative pivot value may be valid for some
                nonlinear and multiphysics analyses (for example,
                electromagnetic and thermal analyses); this key has no
                effect in these cases.

        WARN - Check for negative or zero pivot values for analyses
               performed with the sparse and PCG solvers. When one is
               encountered, a warning is issued and the analysis
               continues. A negative pivot value may be valid for some
               nonlinear and multiphysics analyses (for example,
               electromagnetic and thermal analyses); this key has no
               effect in these cases.

        OFF - Pivot values are not checked. This key causes the
              analysis to continue in spite of a negative or zero
              pivot value.

    prntcntrl
        Provides print options. Print output with these options will be
        sent to the default output file, not to the files created by the
        nonlinear diagnostic tools (NLDIAG).

        ONCE - Print only the maximum and minimum pivot information on
               the first call to the sparse solver (which is the
               default solver). This is the default behavior.

        EVERY - Print the maximum and minimum pivot information at
                every call to the sparse solver. This option is
                provided for nonlinear analysis diagnostics.

    Notes
    -----
    This command is valid for all analyses. In a nonlinear analysis, a
    negative pivot may be valid. In some cases, rigid body motions in
    a nonlinear analysis will be trapped by error routines checking
    infinitely large displacements (DOF limit exceeded) or
    nonconvergence status. An under-constrained model may avoid the
    pivot check, but fail with a DOF limit exceeded error.

    Machine precision may affect whether a small pivot triggers an
    error or bypasses this checking logic. You may wish to review the
    ratio of the maximum to absolute minimum pivot values. For ratios
    exceeding 12 to 14 orders of magnitude, the accuracy of the
    computed solution may be degraded by the severe ill-conditioning
    of the assembled matrix.

    Note that negative pivots corresponding to Lagrange multiplier
    based mixed u-P elements are not checked or reported by this
    command.  Negative pivots arising from the u-P element formulation
    and related analyses can occur and lead to correct solutions.

    This command is also valid in PREP7.
    """
    command = f"PIVCHECK,{key},{prntcntrl}"
    return self.run(command, **kwargs)


def psdcom(self, signif="", comode="", forcetype="", **kwargs):
    """Specifies the power spectral density mode combination method.

    APDL Command: PSDCOM

    Parameters
    ----------
    signif
        Combine only those modes whose significance level exceeds theSIGNIF
        threshold.  For PSD response (SPOPT,PSD), the significance level is
        defined as the modal covariance matrix term, divided by the maximum
        modal covariance matrix term.  Any term whose significance level is
        less than SIGNIF is considered insignificant and is not contributed
        to the mode combinations.  The higher the SIGNIF threshold, the
        fewer the number of terms used. SIGNIF defaults to 0.0001.  If
        SIGNIF is specified as 0.0, it is taken as 0.0.

    comode
        First COMODE number of modes to be actually combined.  COMODE must
        always be less than or equal to NMODE (input quantity NMODE on the
        SPOPT command).  COMODE defaults to NMODE.  COMODE performs a
        second level of control for the first sequential COMODE number of
        modes to be combined.  It uses the significance level threshold
        indicated by SIGNIF and operates only on the significant modes.

    forcetype
        Label identifying the forces to be combined:

        STATIC - Combine the modal static forces (default).

        TOTAL - Combine the modal static plus inertial forces.

    Notes
    -----
    This command is also valid for PREP7.   This command is valid only for
    SPOPT,PSD.

    PSDCOM is not allowed in ANSYS Professional.
    """
    command = f"PSDCOM,{signif},{comode},{forcetype}"
    return self.run(command, **kwargs)


def psdfrq(self, tblno1="", tblno2="", freq1="", freq2="", freq3="",
           freq4="", freq5="", freq6="", freq7="", **kwargs):
    """Defines the frequency points for the input spectrum tables PSDVAL vs.

    APDL Command: PSDFRQ
    PSDFRQ for PSD analysis.

    Parameters
    ----------
    tblno1
        Input table number.  When used with the COVAL or the QDVAL command,
        TBLNO1 represents the row number of this table. Up to 200 tables
        may be defined.

    tblno2
        Input table number.  TBLNO2 is used only for the COVAL or the QDVAL
        commands and represents the column number of this table.

    freq1, freq2, freq3, . . . , freq7
        Frequency points (cycles/time) for spectrum vs. frequency tables.
        FREQ1 should be greater than zero, and values must be in ascending
        order.  Log-log interpolation will be used between frequency
        points.

    Notes
    -----
    The spectrum values may be input with the PSDVAL, COVAL , or QDVAL
    commands.  A separate PSDFRQ command must be used for each table and
    cross table defined.  Frequencies must be in ascending order.

    Repeat PSDFRQ command for additional frequency points.  Values are
    added after the last nonzero frequency.  If all fields after PSDFRQ are
    blank, all input vs. frequency tables are erased.  If TBLNO1 is
    nonblank, all corresponding PSDVAL tables are erased.  If both TBLNO1
    and TBLNO2 are nonblank,  all corresponding COVAL and QDVAL tables are
    erased.

    This command is also valid in PREP7.
    """
    command = f"PSDFRQ,{tblno1},{tblno2},{freq1},{freq2},{freq3},{freq4},{freq5},{freq6},{freq7}"
    return self.run(command, **kwargs)


def psdgraph(self, tblno1="", tblno2="", **kwargs):
    """Displays input PSD curves

    APDL Command: PSDGRAPH

    Parameters
    ----------
    tblno1
        PSD table number to display.

    tblno2
        Second PSD table number to display. TBLNO2 is used only in
        conjunction with the COVAL or the QDVAL commands.

    Notes
    -----
    The input PSD tables are displayed in log-log format as dotted lines.
    The best-fit curves, used to perform the closed-form integration, are
    displayed as solid lines. If there is a significant discrepancy between
    the two, then you should add one or more intermediate points to the
    table to obtain a better fit.

    If TBLNO2 is zero, blank, or equal to TBLNO1, then the autospectra
    (PSDVAL) are displayed for TBLNO1. If TBLNO2 is also specified, then
    the autospectra for TBLNO1 and TBLNO2 are displayed, along with the
    corresponding cospectra (COVAL) and quadspectra (QDVAL), if they are
    defined.

    This command is valid in any processor.
    """
    command = f"PSDGRAPH,{tblno1},{tblno2}"
    return self.run(command, **kwargs)


def psdres(self, lab="", relkey="", **kwargs):
    """Controls solution output written to the results file from a PSD

    APDL Command: PSDRES
    analysis.

    Parameters
    ----------
    lab
        Label identifying the solution output:

        DISP - Displacement solution (default).  One-sigma displacements, stresses, forces,
               etc.  Written as load step 3 on File.RST.

        VELO - Velocity solution.  One-sigma velocities, "stress velocities," "force
               velocities," etc.  Written as load step 4 of File.RST.

        ACEL - Acceleration solution.  One-sigma accelerations, "stress accelerations," "force
               accelerations," etc.  Written as load step 5 on File.RST.

    relkey
        Key defining relative or absolute calculations:

        REL - Calculations are relative to the base excitation (default).

        ABS - Calculations are absolute.

        OFF - No calculation of solution output identified by Lab.

    Notes
    -----
    Controls the amount and form of solution output written to the results
    file from a PSD analysis.  One-sigma values of the relative or absolute
    displacement solution, relative or absolute velocity solution, relative
    or absolute acceleration solution, or any combination may be included
    on the results file.

    This command is also valid in PREP7.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"PSDRES,{lab},{relkey}"
    return self.run(command, **kwargs)


def psdspl(self, tblno="", rmin="", rmax="", **kwargs):
    """Defines a partially correlated excitation in a PSD analysis.

    APDL Command: PSDSPL

    Parameters
    ----------
    tblno
        Input PSD table number defined with PSDVAL command.

    rmin
        Minimum distance between excitation points which are partially
        correlated. Excited nodes closer than RMIN will be fully
        correlated.

    rmax
        Maximum distance between excitation points which are partially
        correlated. Excited nodes farther apart than RMAX will be
        uncorrelated.

    Notes
    -----
    Defines a partially correlated excitation in terms of a sphere of
    influence relating excitation point geometry (in a PSD analysis).  If
    the distance between any two excitation points is less than RMIN, then
    the excitation is fully correlated.  If the distance is greater than
    RMAX, then the excitation is uncorrelated.  If the distance lies
    between RMIN and RMAX, then the excitation is partially correlated with
    the degree of correlation dependent on the separation distance between
    the points. This command is not available for a pressure PSD analysis.

    This command is also valid in PREP7.
    """
    command = f"PSDSPL,{tblno},{rmin},{rmax}"
    return self.run(command, **kwargs)


def psdunit(self, tblno="", type_="", gvalue="", **kwargs):
    """Defines the type of input PSD.

    APDL Command: PSDUNIT

    Parameters
    ----------
    tblno
        Input table number.

    type\_
        Label identifying the type of spectrum:

        DISP - Displacement spectrum (in terms of displacement2/Hz ).

        VELO - Velocity spectrum (in terms of velocity2/Hz ).

        ACEL - Acceleration spectrum (in terms of acceleration2/Hz ).

        ACCG - Acceleration spectrum (in terms of g2/Hz ).

        FORC - Force spectrum (in terms of force2/Hz ).

        PRES - Pressure spectrum (in terms of pressure2/Hz ).

    gvalue
        Value of acceleration due to gravity in any arbitrary units for
        Type=ACCG.  Default is 386.4 in/sec2.

    Notes
    -----
    Defines the type of PSD defined by the PSDVAL, COVAL, and QDVAL
    commands.

    Force (FORC) and pressure (PRES) type spectra can be used only as a
    nodal excitation.

    GVALUE is valid only when type ACCG is specified.  A zero or negative
    value cannot be used.  A parameter substitution can also be performed.

    This command is also valid in PREP7.
    """
    command = f"PSDUNIT,{tblno},{type_},{gvalue}"
    return self.run(command, **kwargs)


def psdval(self, tblno="", sv1="", sv2="", sv3="", sv4="", sv5="", sv6="",
           sv7="", **kwargs):
    """Defines PSD values.

    APDL Command: PSDVAL

    Parameters
    ----------
    tblno
        Input table number being defined.

    sv1, sv2, sv3, . . . , sv7
        Spectral values corresponding to the frequency points [PSDFRQ].
        Values are interpreted as defined with the PSDUNIT command.

    Notes
    -----
    Defines PSD values to be associated with the previously defined
    frequency points.

    Repeat PSDVAL command for additional values, up to the number of
    frequency points [PSDFRQ].  Values are added after the last nonzero
    value.

    This command is also valid in PREP7.
    """
    command = f"PSDVAL,{tblno},{sv1},{sv2},{sv3},{sv4},{sv5},{sv6},{sv7}"
    return self.run(command, **kwargs)


def psdwav(self, tblno="", vx="", vy="", vz="", **kwargs):
    """Defines a wave propagation excitation in a PSD analysis.

    APDL Command: PSDWAV

    Parameters
    ----------
    tblno
        Input PSD table number defined with PSDVAL command.

    vx
        Global Cartesian X-velocity of traveling wave.

    vy
        Global Cartesian Y-velocity of traveling wave.

    vz
        Global Cartesian Z-velocity of traveling wave.

    Notes
    -----
    Defines a traveling wave in a PSD analysis. This command is not
    available for a pressure PSD analysis.

    This command is also valid in PREP7.
    """
    command = f"PSDWAV,{tblno},{vx},{vy},{vz}"
    return self.run(command, **kwargs)


def qdval(self, tblno1="", tblno2="", sv1="", sv2="", sv3="", sv4="",
          sv5="", sv6="", sv7="", **kwargs):
    """Defines PSD quadspectral values.

    APDL Command: QDVAL

    Parameters
    ----------
    tblno1
        First input PSD table number associated with this spectrum.

    tblno2
        Second input PSD table number associated with this spectrum.

    sv1, sv2, sv3, . . . , sv7
        PSD quadspectral values corresponding to the frequency points
        [PSDFRQ].

    Notes
    -----
    Defines PSD quadspectral values to be associated with the previously
    defined frequency points.  Repeat QDVAL command with the same table
    number for additional points.  Unlike autospectra [PSDVAL], the
    quadspectra can be positive or negative.  The quadspectral curve
    segment where there is a sign change is interpolated linearly (the rest
    of the curve segments use log-log interpolation).  For better accuracy,
    choose as small a curve segment as possible wherever a sign change
    occurs.

    Two table numbers are required since values are off-diagonal terms.
    This command is valid for SPOPT,PSD only.

    This command is also valid in PREP7.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"QDVAL,{tblno1},{tblno2},{sv1},{sv2},{sv3},{sv4},{sv5},{sv6},{sv7}"
    return self.run(command, **kwargs)


def rock(self, cgx="", cgy="", cgz="", omx="", omy="", omz="", **kwargs):
    """Specifies a rocking response spectrum.

    APDL Command: ROCK

    Parameters
    ----------
    cgx, cgy, cgz
        Global Cartesian X, Y, and Z location of center of rotation about
        which rocking occurs.

    omx, omy, omz
        Global Cartesian angular velocity components associated with the
        rocking.

    Notes
    -----
    Specifies a rocking response spectrum effect in the spectrum
    (ANTYPE,SPECTR) analysis.

    The excitation direction with rocking included is not normalized to
    one; rather, it scales the spectrum. For more information, see
    Participation Factors and Mode Coefficients.

    This command is also valid in PREP7.
    """
    command = f"ROCK,{cgx},{cgy},{cgz},{omx},{omy},{omz}"
    return self.run(command, **kwargs)


def rose(self, signif="", label="", td="", forcetype="", **kwargs):
    """Specifies the Rosenblueth mode combination method.

    APDL Command: ROSE

    Parameters
    ----------
    signif
        Combine only those modes whose significance level exceeds the
        SIGNIF threshold. For single point, multipoint, or DDAM response
        (SPOPT, SPRS, MPRS, or DDAM), the significance level of a mode is
        defined as the mode coefficient of the mode, divided by the maximum
        mode coefficient of all modes. Any mode whose significance level is
        less than SIGNIF is considered insignificant and does not
        contribute to the mode combinations. The higher the SIGNIF
        threshold, the fewer the number of modes combined. SIGNIF defaults
        to 0.001. If SIGNIF is specified as 0.0, it is taken as 0.0.

    label
        Label identifying the combined mode solution output.

        DISP - Displacement solution (default). Displacements, stresses, forces, etc., are
               available.

        VELO - Velocity solution. Velocities, "stress velocities," "force velocities," etc.,
               are available.

        ACEL - Acceleration solution. Accelerations, "stress accelerations," "force
               accelerations," etc. are available.

    td
        Time duration for earthquake or shock spectrum. TD defaults to 10.

    forcetype
        Label identifying the forces to be combined:

        STATIC - Combine the modal static forces (default).

        TOTAL - Combine the modal static plus inertial forces.

    Notes
    -----
    For more information on spectrum analysis combination methods, see
    Combination of Modes

    This command is also valid in PREP7.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"ROSE,{signif},{label},{td},{forcetype}"
    return self.run(command, **kwargs)


def rigresp(self, option="", method="", val1="", val2="", **kwargs):
    """Specifies the rigid response calculation.

    APDL Command: RIGRESP

    Parameters
    ----------
    option
        Flag to activate or deactivate the rigid response calculation:

        1 (ON or YES) - Activate.

        2 (OFF or NO) - Deactivate. This value is the default.

    method
        Method used to calculate the rigid response:

        GUPTA - Gupta method.

        LINDLEY - Lindley-Yow method.

    val1
        If Method = GUPTA, Val1 represents the frequency F1 in Hertz.

    val2
        If Method = GUPTA, Val2 represents the frequency F2 in Hertz.

    Notes
    -----
    This rigid response calculation is only valid for single point response
    spectrum analysis (SPOPT, SPRS) and multiple point response spectrum
    analysis (SPOPT, MPRS) with combination methods (SRSS), complete
    quadratic (CQC) or Rosenblueth (ROSE)

    This command is also valid in PREP7.

    Only Sptype = SPRS is allowed in ANSYS Professional.
    """
    command = f"RIGRESP,{option},{method},{val1},{val2}"
    return self.run(command, **kwargs)


def sed(self, sedx="", sedy="", sedz="", cname="", **kwargs):
    """Defines the excitation direction for response spectrum and PSD

    APDL Command: SED
    analyses.

    Parameters
    ----------
    sedx, sedy, sedz
        Global Cartesian coordinates of a point that defines a line
        (through the origin) corresponding to the excitation direction.
        For example: 0.0, 1.0, 0.0 defines global Y as the spectrum
        direction.

    cname
        The component name corresponding to the group of excited nodes.
        Only applies to base excitation multi-point response spectrum
        analysis (SPOPT, MPRS) and power spectral density analysis (SPOPT,
        PSD). Defaults to no component.

    Notes
    -----
    In single-point response spectrum analysis (SPOPT,SPRS), the excitation
    direction without rocking (ROCK) is normalized to one so that the SEDX,
    SEDY, and SEDZ values do not scale the spectrum.  The excitation
    direction with rocking is not normalized. The SEDX, SEDY, and SEDZ
    values must be consistent with the OMX, OMY, and OMZ values on the ROCK
    command. The calculated direction then scales the spectrum. For more
    information, see Participation Factors and Mode Coefficients.

    In multi-point response spectrum analysis (SPOPT,MPRS) and power
    spectral density analysis (SPOPT,PSD), the excitation direction is
    normalized to one so that the SEDX, SEDY, and SEDZ values do not scale
    the spectrum.  The component name (Cname) is required. The constraints
    corresponding to the excitation direction are applied to the component
    nodes.

    This command is also valid in PREP7.
    """
    command = f"SED,{sedx},{sedy},{sedz},{cname}"
    return self.run(command, **kwargs)


def spdamp(self, tblno="", curvno="", dampratio="", **kwargs):
    """Defines input spectrum damping in a multi-point response spectrum

    APDL Command: SPDAMP
    analysis.

    Parameters
    ----------
    tblno
        Input table number. Corresponds to the frequency table number
        (TBLNO on the SPFREQ command).

    curvno
        Input curve number. Corresponds to the spectrum values curve number
        (CURVNO on the SPVAL command).

    dampratio
        Damping ratio for the response spectrum curve. Up to 20 different
        curves may be defined, each with a different damping ratio. Damping
        values must be input in ascending order.

    Notes
    -----
    Defines multi-point response spectrum damping value to be associated
    with:

    Previously defined frequency points (SPFREQ).

    Subsequently defined spectrum points (SPVAL).

    Damping values are used only to identify input spectrum values for the
    mode coefficients calculation.

    The curve number must be input in ascending order starting with 1.

    This command is also valid in PREP7.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"SPDAMP,{tblno},{curvno},{dampratio}"
    return self.run(command, **kwargs)


def spfreq(self, tblno="", freq1="", freq2="", freq3="", freq4="",
           freq5="", freq6="", freq7="", **kwargs):
    """Defines the frequency points for the input spectrum tables SPVAL vs.

    APDL Command: SPFREQ
    SPFREQ for multi-point spectrum analysis.

    Parameters
    ----------
    tblno
        Input table number. Up to 200 tables may be defined.

    freq1, freq2, freq3,..., freq7
        Frequency points (Hz) for spectrum vs. frequency tables. FREQ1
        should be greater than zero, and values must be in ascending order.

    Notes
    -----
    The spectrum values are input with the SPVAL command. A separate SPFREQ
    command must be used for each table defined. Frequencies must be in
    ascending order.

    Repeat SPFREQ command for additional frequency points. Values are added
    after the last nonzero frequency.

    If all fields after SPFREQ are blank, all input vs. frequency tables
    are erased. If TBLNO is the only non-blank field, all corresponding
    SPVAL curves are erased.

    Use the SPTOPT and STAT commands to list current frequency points.

    This command is also valid in PREP7.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"SPFREQ,{tblno},{freq1},{freq2},{freq3},{freq4},{freq5},{freq6},{freq7}"
    return self.run(command, **kwargs)


def spgraph(self, tblno="", curvno="", curvnobeg="", **kwargs):
    """Displays input spectrum curves for MPRS analysis.

    APDL Command: SPGRAPH

    Parameters
    ----------
    tblno
        Table number to display.  Defaults to 1.

    curvno
        Curve number to display. Defaults to none.

    curvnobeg
        Beginning of the curve number range to display. Defaults to 1.

    Notes
    -----
    You can display up to 10 input spectrum curves (SPVAL and SPFREQ
    commands) with log X scale.

    If the input spectrum curves are not associated with a damping value
    (SPDAMP command), CURVNO and CURVNOBeg are not applicable and table
    TBLNO is displayed. Otherwise, specify CURVNO or CURVNOBeg:

    if CURVNO is used, one curve is displayed.

    if CURVNOBeg is used, up to 10 curves are displayed. CURVNOBeg is the
    beginning of the curve number range of interest.
    """
    command = f"SPGRAPH,{tblno},{curvno},{curvnobeg}"
    return self.run(command, **kwargs)


def spopt(self, sptype="", nmode="", elcalc="", modereusekey="", **kwargs):
    """Selects the spectrum type and other spectrum options.

    APDL Command: SPOPT

    Parameters
    ----------
    sptype
        Spectrum type:

        SPRS - Single point excitation response spectrum (default).  See also the SVTYP
               command.

        MPRS - Multiple point excitation response spectrum.

        DDAM - Dynamic design analysis method.

        PSD - Power spectral density.

    nmode
        Use the first NMODE modes from the modal analysis.  Defaults to all
        extracted modes, as specified by the MODOPT and BUCOPT commands.
        NMODE cannot be larger than 10000.

    elcalc
        Element results calculation key (for Sptype = PSD only):

        NO - Do not calculate element results and reaction forces (default).

        YES - Calculate element results and reaction forces, as well as the nodal degree of
              freedom solution.

    modereusekey
        Key for existing MODE file reuse when running multiple spectrum
        analyses:

        NO - No spectrum analysis has been performed yet (default).

        YES - This is not the first spectrum analysis.  The MODE file will be reused and the
              necessary files will be cleaned up for the new spectrum
              analysis.

    Notes
    -----
    Valid only for a spectrum analysis (ANTYPE,SPECTR).  This operation
    must be preceded by a modal solution (ANTYPE,MODAL) with the
    appropriate files available.  Both the spectrum analysis and the
    preceding modal analysis must be performed under the same ANSYS version
    number.

    If used in SOLUTION, this command is valid only within the first load
    step.

    This command is also valid in PREP7.

    Only Sptype = SPRS is allowed in ANSYS Professional.
    """
    command = f"SPOPT,{sptype},{nmode},{elcalc},{modereusekey}"
    return self.run(command, **kwargs)


def spunit(self, tblno="", type_="", gvalue="", keyinterp="", **kwargs):
    """Defines the type of multi-point response spectrum.

    APDL Command: SPUNIT

    Parameters
    ----------
    tblno
        Input table number.

    type\_
        Label identifying the type of spectrum:

        DISP   - Displacement spectrum (SPVAL values interpreted as displacements with units of
                 length).

        VELO   - Velocity spectrum (SPVAL values interpreted as velocities with units of
                 length/time).

        ACEL   - Acceleration spectrum (SPVAL values interpreted as accelerations with units of
                 length/time2).

        ACCG   - Acceleration spectrum (SPVAL values interpreted as accelerations with units of
                 g/time2).

        FORC   - Force spectrum.

        PRES   - Pressure spectrum.

    gvalue
        Value of acceleration due to gravity in any arbitrary units for
        Type=ACCG table. Default is 386.4 in/sec2.

    keyinterp
        Key to activate or deactivate the linear interpolation between
        input response spectrum points and input response spectrum curves:

        0 (OFF or NO) - Deactivate linear and use logarithmic interpolation. This value is the default.

        1 (ON or YES) - Activate linear interpolation.

    Notes
    -----
    Defines the type of multi-point response spectrum defined by the SPFREQ
    and  SPVAL commands.

    Force (FORC) and pressure (PRES) type spectra can be used only as a
    nodal excitation.

    GVALUE is valid only when type=ACCG is specified. A zero or negative
    value cannot be used. A parameter substitution can also be performed.

    This command is also valid in PREP7.
    """
    command = f"SPUNIT,{tblno},{type_},{gvalue},{keyinterp}"
    return self.run(command, **kwargs)


def spval(self, tblno="", curvno="", sv1="", sv2="", sv3="", sv4="",
          sv5="", sv6="", sv7="", **kwargs):
    """Defines multi-point response spectrum values.

    APDL Command: SPVAL

    Parameters
    ----------
    tblno
        Input table number. It corresponds to TBLNO on the SPFREQ command.

    curvno
        Input curve number. It corresponds to CURVNO on the SPDAMP command
        (optional).

    sv1, sv2, sv3, , , . . . , sv7
        Spectral values corresponding to the frequency points (SPFREQ) and
        damping ratio (SPDAMP). Values are interpreted as defined with the
        SPUNIT command.

    Notes
    -----
    Defines multi-point response spectrum values to be associated with the
    previously defined frequency points (SPFREQ). It can also be associated
    with the previously defined damping value (SPDAMP). If CURVNO is not
    specified, the input spectrum is not associated with a damping value.

    Repeat SPVAL command for additional values, up to the number of
    frequency points (SPFREQ). Values are added after the last nonzero
    value.

    The interpolation method between response spectrum points and curves is
    specified using KeyInterp on the SPUNIT command. It is logarithmic by
    default.

    Use the SPTOPT and STAT commands to list current spectrum curve values.

    This command is also valid in PREP7.
    """
    command = f"SPVAL,{tblno},{curvno},{sv1},{sv2},{sv3},{sv4},{sv5},{sv6},{sv7}"
    return self.run(command, **kwargs)


def srss(self, signif="", label="", abssumkey="", forcetype="", **kwargs):
    """Specifies the square root of sum of squares mode combination method.

    APDL Command: SRSS

    Parameters
    ----------
    signif
        Combine only those modes whose significance level exceeds the
        SIGNIF threshold.  For single point, multipoint, or DDAM response
        (SPOPT,SPRS, MPRS or DDAM), the significance level of a mode is
        defined as the mode coefficient of the mode, divided by the maximum
        mode coefficient of all modes.  Any mode whose significance level
        is less than SIGNIF is considered insignificant and is not
        contributed to the mode combinations.  The higher the SIGNIF
        threshold, the fewer the number of modes combined. SIGNIF defaults
        to 0.001.  If SIGNIF is specified as 0.0, it is taken as 0.0.
        (This mode combination method is not valid for SPOPT,PSD.)

    label
        Label identifying the combined mode solution output.

        DISP - Displacement solution (default).  Displacements, stresses, forces, etc., are
               available.

        VELO - Velocity solution.  Velocities, "stress velocities," "force velocities," etc.,
               are available.

        ACEL - Acceleration solution.  Accelerations, "stress accelerations," "force
               accelerations," etc., are available.

    abssumkey
        Absolute Sum combination key (for SPOPT,MPRS only):

        NO - Do not use the Absolute Sum method (default).

        YES - Combine the modes per excitation direction using the Absolute Sum method, then
              combine the resulting quantities using the square root of sum
              of squares method.

    forcetype
        Label identifying the forces to be combined:

        STATIC - Combine the modal static forces (default).

        TOTAL - Combine the modal static plus inertial forces.

    Notes
    -----
    This command is also valid for PREP7.
    """
    command = f"SRSS,{signif},{label},{abssumkey},{forcetype}"
    return self.run(command, **kwargs)


def sv(self, damp="", sv1="", sv2="", sv3="", sv4="", sv5="", sv6="",
       sv7="", sv8="", sv9="", **kwargs):
    """Defines spectrum values to be associated with frequency points.

    APDL Command: SV

    Parameters
    ----------
    damp
        Damping ratio for this response spectrum curve.  If the same as a
        previously defined curve, the SV values are added to the previous
        curve.  Up to four different curves may be defined, each with a
        different damping ratio.  Damping values must be input in ascending
        order.

    sv1, sv2, sv3, . . . , sv9
        Spectrum values corresponding to the frequency points [FREQ].
        Values are interpreted as defined with the SVTYP command.   SV
        values should not be zero.  Values required outside the frequency
        range use the extreme input values.

    Notes
    -----
    Defines the spectrum values to be associated with the previously
    defined frequency points [FREQ].  Applies only to the single-point
    response spectrum.  Damping has no effect on the frequency solution.
    Damping values are used only to identify SV curves for the mode
    combinations calculation.  Only the curve with the lowest damping value
    is used in the initial mode coefficient calculation.  Use STAT command
    to list current spectrum curve values.

    Repeat SV command for additional SV points (100 maximum per DAMP
    curve).  SV values are added to the DAMP curve after the last nonzero
    SV value.

    The interpolation method between response spectrum points and curves is
    specified using KeyInterp in the SVTYP command. It is logarithmic by
    default.

    This command is also valid in PREP7.
    """
    command = f"SV,{damp},{sv1},{sv2},{sv3},{sv4},{sv5},{sv6},{sv7},{sv8},{sv9}"
    return self.run(command, **kwargs)


def svplot(self, optionscale="", damp1="", damp2="", damp3="", damp4="",
           **kwargs):
    """Displays input spectrum curves.

    APDL Command: SVPLOT

    Parameters
    ----------
    optionscale
        Flag to activate or deactivate input spectrum value scaling:

        OFF  - Do not scale the input spectrum values with scale factor FACT (SVTYP command).
               This is the default value.

        ON  - Scale the input spectrum values with scale factor FACT (SVTYP command)

    damp1
        Damping ratio corresponding to DAMP (SV command) defining the first
        spectrum curve.

    damp2
        Damping ratio corresponding to DAMP (SV command) defining the
        second spectrum curve.

    damp3
        Damping ratio corresponding to DAMP (SV command) defining the third
        spectrum curve.

    damp4
        Damping ratio corresponding to DAMP (SV command) defining the
        fourth spectrum curve.

    Notes
    -----
    You can display up to four input spectrum tables (SV and FREQ commands)
    with log X scale. If no damping ratio is specified, all spectrum tables
    are displayed.

    This command is valid in any processor.
    """
    command = f"SVPLOT,{optionscale},{damp1},{damp2},{damp3},{damp4}"
    return self.run(command, **kwargs)


def svtyp(self, ksv="", fact="", keyinterp="", **kwargs):
    """Defines the type of single-point response spectrum.

    APDL Command: SVTYP

    Parameters
    ----------
    ksv
        Response spectrum type:

        0 - Seismic velocity response spectrum loading (SV values interpreted as velocities
            with units of length/time).

        1 - Force response spectrum loading (SV values interpreted as force amplitude
            multipliers).

        2 - Seismic acceleration response spectrum loading (SV values interpreted as
            accelerations with units of length/time2).

        3 - Seismic displacement response spectrum loading (SV values interpreted as
            displacements with units of length).

        4 - PSD loading (SV values interpreted as acceleration2/(cycles/time), such as
            (in/sec2)2/Hz (not g2/Hz)).  (Not recommended)

    fact
        Scale factor applied to spectrum values (defaults to 1.0).  Values
        are scaled when the solution is initiated [SOLVE].  Database values
        remain the same.

    keyinterp
        Key to activate or deactivate the linear interpolation between
        input response spectrum points and input response spectrum curves:

        0 (OFF or NO) - Deactivate linear and use logarithmic interpolation. This value is the default.

        1 (ON or YES) - Activate linear interpolation.

    Notes
    -----
    Defines the type of single-point response spectrum [SPOPT].  The
    seismic excitation direction is defined with the SED command.

    This command is also valid in PREP7.
    """
    command = f"SVTYP,{ksv},{fact},{keyinterp}"
    return self.run(command, **kwargs)


def vddam(self, vf="", va="", vb="", vc="", **kwargs):
    """Specifies the velocity spectrum computation constants for the analysis

    APDL Command: VDDAM
    of shock resistance of shipboard structures.

    Parameters
    ----------
    vf
        Direction-dependent velocity coefficient for elastic or elastic-
        plastic analysis option (Default = 0).

    va, vb, vc
        Coefficients for the DDAM velocity spectrum equations.  See Dynamic
        Design Analysis Method in the Mechanical APDL Theory Reference.
        Default for these coefficients is zero.

    Notes
    -----
    This command specifies velocity coefficients to analyze shock
    resistance of shipboard equipment.  These coefficients are used to
    compute mode coefficients according to the equations given in Dynamic
    Design Analysis Method in the Mechanical APDL Theory Reference.  The
    form of these equations is based on the Naval NRL Dynamic Design
    Analysis Method.  This command, along with the ADDAM and SED commands,
    is used with the spectrum (ANTYPE,SPECTR) analysis as a special purpose
    alternative to the SV, FREQ, and SVTYP commands.  The mass and length
    units of the model must be in pounds and inches, respectively.

    DDASPEC may alternatively be used to calculate spectrum coefficients.

    This command is also valid in PREP7.
    """
    command = f"VDDAM,{vf},{va},{vb},{vc}"
    return self.run(command, **kwargs)



import warnings

from typing import Optional, Union
from ansys.mapdl.core.mapdl_types import MapdlInt, MapdlFloat


def cplane(self, key="", **kwargs):
    """Specifies the cutting plane for section and capped displays.

    APDL Command: /CPLANE

    Parameters
    ----------
    key
        Specifies the cutting plane:

        0 - Cutting plane is normal to the viewing vector [/VIEW] and passes through the
            focus point [/FOCUS] (default).

        1 - The working plane [WPLANE] is the cutting plane.

    Notes
    -----
    Defines the cutting plane to be used for section and capped displays
    [/TYPE,,(1, 5, or 7)].

    This command is valid in any processor.
    """
    command = f"/CPLANE,{key}"
    return self.run(command, **kwargs)


def ctype(self, key="", dotd="", dots="", dshp="", tlen="", **kwargs):
    """Specifies the type of contour display.

    APDL Command: /CTYPE

    Parameters
    ----------
    key
        Type of display:

        0 - Standard contour display.

        1 - Isosurface display.

        2 - Particle gradient display.

        3 - Gradient triad display.

    dotd
        Maximum dot density for particle gradient display (KEY = 2).
        Density is expressed as dots per screen width (defaults to 30).

    dots
        Dot size for particle gradient display (KEY = 2).  Size is
        expressed as a fraction of the screen width (defaults to 0.0
        (single dot width)).

    dshp
        Spherical dot shape precision for particle gradient display (KEY =
        2).  (3-D options are supported only on 3-D devices):

        0 - Flat 2-D circular dot.

        1 - Flat-sided 3-D polyhedron.

        n - 3-D sphere with n (>1) polygon divisions per 90° of radius.

    tlen
        Maximum length of triads for gradient triad display (KEY = 3).
        Value is expressed as a fraction of the screen width (defaults to
        0.067).

    Notes
    -----
    Use /CTYPE,STAT to display the current settings.     Only the standard
    contour display [/CTYPE,0) and the isosurface contour display
    [/CTYPE,1] are supported by PowerGraphics [/GRAPHICS,POWER].

    This command is valid in any processor.
    """
    command = f"/CTYPE,{key},{dotd},{dots},{dshp},{tlen}"
    return self.run(command, **kwargs)


def edge(self, wn="", key="", angle="", **kwargs):
    """Displays only the common lines ("edges") of an object.

    APDL Command: /EDGE

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies. The default window
        is 1.

    key
        Edge key:

        0 - Display common lines between all adjacent element faces.

        1 - Display only the common lines between non-coplanar faces (that is, show only
            the edges).

    angle
        Largest angle between two faces for which the faces are considered
        to be coplanar (0° to 180°). Defaults to 45°. A smaller angle
        produces more edges, a larger angle produces fewer edges.

    Notes
    -----
    The ANGLE field is used in PowerGraphics to determine geometric
    discontinuities. It is a tolerance measure for the differences between
    the normals of the surfaces being considered. Values within the
    tolerance are accepted as coplanar (geometrically continuous). In
    postprocessing displays, results are not averaged across discontinuous
    surfaces.

    A surface can be displayed as an edge outline without interior detail.
    This is useful for both geometry and postprocessing displays. Element
    outlines are normally shown as solid lines for geometry and
    displacement displays. Lines common to adjacent "coplanar" element
    faces are removed from the display.  Midside nodes of elements are
    ignored.

    The /SHRINK option is ignored with the edge option.

    /EDGE is not supported for PLESOL and /ESHAPE displays when in
    PowerGraphics mode (/GRAPHICS,POWER).

    The /EDGE command is valid in any processor.
    """
    command = f"/EDGE,{wn},{key},{angle}"
    return self.run(command, **kwargs)


def eshape(self, scale: Union[str, int] = "", key: MapdlInt = "",
           **kwargs) -> Optional[str]:
    """Displays elements with shapes determined from the real constants or section definition.

    APDL Command: /ESHAPE


    Parameters
    ----------
    scale
        Scaling factor:

        * 0 - Use simple display of line and area elements. This
          value is the default.

        * 1 - Use real constants or section definition to form a
            solid shape display of the applicable elements.

        FAC - Multiply certain real constants, such as thickness,
              by FAC (where FAC > 0.01) and use them to form a
              solid shape display of elements.

    key
        Current shell thickness key:

        * 0 - Use current thickness in the displaced solid shape
              display of shell elements (valid for SHELL181,
              SHELL208, SHELL209, and SHELL281). This value is the
              default.

        * 1 - Use initial thickness in the displaced solid shape
          display of shell elements.

    Notes
    -----
    The /ESHAPE command allows beams, shells, current sources, and
    certain special-purpose elements to be displayed as solids
    with the shape determined from the real constants or section
    types. Elements are displayed via the EPLOT command. No checks
    for valid or complete input are made for the display.

    Following are details about using this command with various
    element types:

    SOLID65 elements are displayed with internal lines that
    represent rebar sizes and orientations (requires vector mode
    [/DEVICE] with a basic type of display [/TYPE,,BASIC]). The
    rebar with the largest volume ratio in each element plots as a
    red line, the next largest as green, and the smallest as blue.

    COMBIN14, COMBIN39, and MASS21 are displayed with a graphics
    icon, with the offset determined by the real constants and
    KEYOPT settings.

    BEAM188, BEAM189, PIPE288, PIPE289 and ELBOW290 are displayed
    as solids with the shape determined via the section-definition
    commands (SECTYPE and SECDATA). The arbitrary section option
    (Subtype = ASEC) has no definite shape and appears as a thin
    rectangle to show orientation. The elements are displayed with
    internal lines representing the cross- section mesh.

    SOLID272 and SOLID273 are displayed as solids with the shape
    determined via the section-definition commands (SECTYPE and
    SECDATA).  The 2-D master plane is revolved around the
    prescribed axis of symmetry.

    Contour plots are available for these elements in
    postprocessing for PowerGraphics only (/GRAPHICS,POWER). To
    view 3-D deformed shapes for the elements, issue OUTRES,MISC
    or OUTRES,ALL for static or transient analyses. To view 3-D
    mode shapes for a modal or eigenvalue buckling analysis,
    expand the modes with element results calculation ON (Elcalc =
    YES for MXPAND).

    SOURC36, CIRCU124, and TRANS126 elements always plot using
    /ESHAPE when PowerGraphics is activated (/GRAPHICS,POWER).

    In most cases, /ESHAPE renders a thickness representation of
    your shell, plane and layered elements more readily in
    PowerGraphics (/GRAPHICS,POWER). This type of representation
    employs PowerGraphics to generate the enhanced representation,
    and will often provide no enhancement in Full Graphics
    (/GRAPHICS,FULL). This is especially true for POST1 results
    displays, where /ESHAPE is not supported for most element
    types with FULL graphics.

    When PowerGraphics is active, /ESHAPE may degrade the image if
    adjacent elements have overlapping material, such as shell
    elements which are not co-planar. Additionally, if adjacent
    elements have different thicknesses, the polygons depicting
    the connectivity between the "thicker" and "thinner" elements
    along the shared element edges may not always be displayed.

    For POST1 results displays (such as PLNSOL), the following
    limitations apply:

    Rotational displacements for beam elements are used to create
    a more realistic displacement display. When /ESHAPE is active,
    displacement plots (via PLNSOL,U,X and PLDISP, for example)
    may disagree with your PRNSOL listings. This discrepancy will
    become more noticeable when the SCALE value is not equal to
    one.

    When shell elements are not co-planar, the resulting PLNSOL
    display with /ESHAPE will actually be a PLESOL display as the
    non-coincident pseudo-nodes are not averaged. Additionally,
    /ESHAPE should not be used with coincident elements because
    the plot may incorrectly average the displacements of the
    coincident elements.

    When nodes are initially coincident and PowerGraphics is
    active, duplicate polygons are eliminated to conserve display
    time and disk space. The command may degrade the image if
    initially coincident nodes have different displacements. The
    tolerance for determining coincidence is 1E-9 times the
    model’s bounding box diagonal.

    If you want to view solution results (PLNSOL, etc.) on layered
    elements (such as SHELL181, SOLSH190, SOLID185 Layered Solid,
    SOLID186 Layered Solid, SHELL208, SHELL209, SHELL281, and
    ELBOW290), set KEYOPT(8) = 1 for the layer elements so that
    the data for all layers is stored in the results file.

    You can plot the through-thickness temperatures of elements
    SHELL131 and SHELL132 regardless of the thermal DOFs in use by
    issuing the PLNSOL,TEMP command (with PowerGraphics and
    /ESHAPE active).

    The /ESHAPE,1 and /ESHAPE,FAC commands are incompatible with
    the /CYCEXPAND command used in cyclic symmetry analyses.

    This command is valid in any processor.

    """
    warnings.warn('pymapdl does not support /ESHAPE when plotting in '
                  'Python using ``mapdl.eplot()``.  '
                  'Use ``mapdl.eplot(vtk=False)`` ')
    command = f"/ESHAPE,{scale},{key}"
    return self.run(command, **kwargs)


def facet(self, lab="", **kwargs):
    """Specifies the facet representation used to form solid model displays.

    APDL Command: /FACET

    Parameters
    ----------
    lab
        Valid labels:

        FINE - Use finer tessellation to increase the number of facets for the display.
               Provides the best representation (but decreases speed of
               operation).

        NORML - Use the basic number of facets for the display (default).

        COAR - Use a limited number of facets for the display. This option will increase the
               speed of the operations, but may produce poor
               representations for some imported models.

        WIRE - Display model with a wireframe representation (fast, but surfaces will not be
               shown).

    Notes
    -----
    Specifies the facet (or polygon) representation used to form solid
    model displays.  Used only with the APLOT, ASUM, VPLOT, and VSUM
    commands.

    This command is valid in any processor.
    """
    command = f"/FACET,{lab}"
    return self.run(command, **kwargs)


def gline(self, wn="", style="", **kwargs):
    """Specifies the element outline style.

    APDL Command: /GLINE

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies (defaults to 1).

    style
        Outline key:

         0  - Solid element outlines (default)

         1  - Dashed element outlines

        -1  - No element outlines

    Notes
    -----
    Determines the element outline style.  Often used when node numbers are
    displayed to prevent element lines from overwriting node numbers.

    Unless you are using an OpenGL or Starbase driver, the dashed element
    outline option (/GLINE,WN,1) is not available in the following
    situations:

    Z-buffered displays (/TYPE,WN,6).

    Capped Z-buffered displays (/TYPE,WN,7).

    Qslice Z-buffered displays (/TYPE,WN,8).

    This command is valid in any processor.
    """
    command = f"/GLINE,{wn},{style}"
    return self.run(command, **kwargs)


def gmarker(self, curve="", key="", incr="", **kwargs):
    """Specifies the curve marking style.

    APDL Command: /GMARKER

    Parameters
    ----------
    curve
        Curve number markers will be applied on (integer value between 1
        and 10).

    key
        Marker key:

        0 - No markers will be applied (default).

        1 - TRIANGLES will be applied.

        2 - SQUARES will be applied.

        3 - DIAMONDS  will be applied.

        4 - CROSSES will be applied.

    incr
        Determines the curve marking frequency. (a whole number value
        between 1 and 255). If INCR = 1, markers are displayed at every
        data point on the curve. If INCR = 2 then markers are displayed at
        every second data point. If INCR = 3 then they are displayed at
        every third data point.

    Notes
    -----
    The user-specified markers will not be drawn when the area under the
    curve is color-filled (/GROPT, FILL).
    """
    command = f"/GMARKER,{curve},{key},{incr}"
    return self.run(command, **kwargs)


def gmface(self, lab="", n="", **kwargs):
    """Specifies the facet representation used to form solid models.

    APDL Command: GMFACE

    Parameters
    ----------
    lab
        Valid Labels:

        FINE - Value that determines how coarse the facets will be.

    n
        An integer value between one (small) and ten (large) that
        determines the tolerances that will be applied to the creation of
        arcs and surfaces. Ten will create many facets, which may in turn
        cause ANSYS to run very slowly. One will create fewer facets, which
        may in turn cause larger tolerance errors.
    """
    command = f"GMFACE,{lab},{n}"
    return self.run(command, **kwargs)


def light(self, wn="", num="", int_="", xv="", yv="", zv="", refl="",
          **kwargs):
    """Specifies the light direction for the display window.

    APDL Command: /LIGHT

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies (defaults to 1).

    num
        Ambient or directional light key:

        0 - Ambient light (default).

        1 - Directional light.

    int\_
        Light intensity factor (defaults to 0.3 for ambient, 1.0 for
        directional). This option is valid only for 3-D devices).

    xv, yv, zv
        Light direction (valid only for NUM = 1).  The directional light
        source is parallel to the line from point XV, YV, ZV to the origin,
        in the global Cartesian system origin.  Defaults to the viewing
        direction [/VIEW].

    refl
        Light reflectance factor (valid only for NUM = 1 and 3-D devices).

    Notes
    -----
    Defines the light direction for the window.  Use this command only with
    3-D graphics devices or 2-D devices when Z-buffering is used [/TYPE,,(6
    or 7)].  The ambient light has no direction, only an intensity.  You
    can position the directional light source  by defining a point (in the
    global Cartesian coordinate system) representing a point along the
    light directional line.  This point, and the global Cartesian
    coordinate system origin, define the line along which the light is
    positioned looking toward the origin.  You can use any point along the
    light line; for example, both (1.,1.,1.) and (2.,2.,2.) give the same
    light effect.  For 3-D graphics devices only, the directional light
    source also has intensity and reflectance factors.

    By choosing the highest intensity ambient light for 3-D graphics
    devices (via the command /LIGHT,WN,0,1), you can nullify color shading
    and other effects of directional lighting.

    This command is valid in any processor.
    """
    command = f"/LIGHT,{wn},{num},{int_},{xv},{yv},{zv},{refl}"
    return self.run(command, **kwargs)


def normal(self, wn="", key="", **kwargs):
    """Allows displaying area elements by top or bottom faces.

    APDL Command: /NORMAL

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies (defaults to 1).

    key
        Display key:

        0 - No face distinction.

        1 - Show only area elements having their positive normals directed toward the
            viewing point.

        -1 - Show only area elements having their positive normals directed away from the
             viewing point.

    Notes
    -----
    /NORMAL allows you to select area elements and area plots by the top or
    bottom faces.  It is useful for checking the normal directions on shell
    elements.  The positive normal (element Z direction) is defined by the
    right-hand rule following the node I, J, K, L input direction.  This
    command is available only with raster or hidden-line displays, for
    WIN32 or X11 2-D displays only.

    This command is valid in any processor.
    """
    command = f"/NORMAL,{wn},{key}"
    return self.run(command, **kwargs)


def shade(self, wn="", type_="", **kwargs):
    """Defines the type of surface shading used with Z-buffering.

    APDL Command: /SHADE

    Parameters
    ----------
    wn
        Window number (or ALL) to which command applies (defaults to 1).

    type\_
        Shading type:

        FACET or 0 - Facet shading (one color per area face) (default).

        GOURAUD or 1 - Gouraud smooth shading (smooth variation of color based on interpolated vertex
                       colors).

        PHONG or 2 - Phong smooth shading (smooth variation of color based on interpolated vertex
                     normals).

    Notes
    -----
    Defines the type of surface shading used on area, volume, and
    PowerGraphics [/GRAPHICS,POWER] displays when software Z-buffering is
    enabled [/TYPE].  This command is only functional for 2-D display
    devices.

    This command is valid in any processor.
    """
    command = f"/SHADE,{wn},{type_}"
    return self.run(command, **kwargs)


def trlcy(self, lab="", tlevel="", n1="", n2="", ninc="", **kwargs):
    """Specifies the level of translucency.

    APDL Command: /TRLCY

    Parameters
    ----------
    lab
        Apply translucency level to the items specified by the following
        labels:

        ELEM - Elements.  Use N1, N2, NINC fields for element numbers.

        AREA - Solid model areas.  Use N1, N2, NINC fields for area numbers.

        VOLU - Solid model volumes.  Use N1, N2, NINC fields for volume numbers.

        ISURF - Isosurfaces (surfaces of constant stress, etc., value).  Translucency varies
                with result value, to a maximum of the specified
                translucency level.

        CM - Component group.  Use N1 for component name, ignore N2 and NINC.

        CURVE - Filled areas under curves of line graphs.  Use N1, N2, NINC fields for curve
                numbers.

        ZCAP - If /TYPE,WN,ZCAP is the current display type, then /TRLCY,ZCAP,TLEVEL will
               display the model in window WN with the portion of the model
               in front of the section plane displayed at the translucency
               level TLEVEL.

        ON, OFF - Sets the specified translucency display on or off. All other fields are
                  ignored.

    tlevel
        Translucency level:  0.0 (opaque) to 1.0 (transparent).

    n1, n2, ninc
        Used only with labels as noted above.  Apply translucency level to
        Lab items numbered N1 to N2 (defaults to N1) in steps of NINC
        (defaults to 1).  If N1 is blank or ALL, apply specified
        translucency level to entire selected range.  If Lab is CM, use
        component name for N1 and ignore N2 and NINC. A value of N1 = P
        allows you to graphically pick elements, areas, and volumes. You
        can then assign translucency levels to the entities via the picker.
        The Lab and TLEVEL fields are ignored when translucency is applied
        by picking.

    Notes
    -----
    Specifies the level of translucency for various items.  Issue
    /TRLCY,DEFA to reset the default (0) translucency levels.  This command
    is valid only on selected 2-D and 3-D graphics devices; see  in the
    Basic Analysis Guide for more information on applying translucency.

    For 2-D devices, ANSYS displays only the visible faces of the items
    being displayed.  The information behind the facing planes is not
    displayed.  Issuing the /SHRINK command will force the hardware to
    display information behind the translucent items.

    This command is valid in any processor.
    """
    command = f"/TRLCY,{lab},{tlevel},{n1},{n2},{ninc}"
    return self.run(command, **kwargs)



from typing import Optional, Union
from ansys.mapdl.core.mapdl_types import MapdlInt, MapdlFloat


def ansol(self, nvar="", node="", item="", comp="", name="", mat="",
          real="", ename="", **kwargs):
    """Specifies averaged nodal data to be stored from the results file in the

    APDL Command: ANSOL
    solution coordinate system.

    Parameters
    ----------
    nvar
        Arbitrary reference number assigned to this variable (2 to NV
        [NUMVAR]). Overwrites any existing results for this variable.

    node
        Node number for which data are to be stored.

    item
        Label identifying the item. General item labels are shown in
        Table 126: ANSOL - General Item and Component Labels below. Some
        items also require a component label.

    comp
        Component of the item (if required). General component labels are
        shown in Table 126: ANSOL - General Item and Component Labels
        below.

    name
        Thirty-two character name for identifying the item on the printout
        and displays. Defaults to an eight character label formed by
        concatenating the first four characters of the Item and Comp
        labels.

    mat
        The material number. Average will be computed based on the subset
        of elements with the specified material number. DEFAULT: Use all
        elements in the active set unless Real and/or Ename is specified.

    real
        The real number. Average will be computed based on the subset of
        elements with the specified real number. DEFAULT: Use all elements
        in the active set unless Mat and/or Ename is specified.

    ename
        The element type name. Average will be computed based on the subset
        of elements with the specified element type name. DEFAULT: Use all
        elements in the active set unless Mat and/or Real is specified.

    Notes
    -----
    Valid item and component labels for averaged nodal results are listed
    in Table: 126:: ANSOL - General Item and Component Labels, below.

    All element nodal quantities are obtained in RSYS, Solu and then
    averaged.

    The ANSOL command defines averaged nodal results data to be stored from
    a results file [FILE]. Not all items are valid for all nodes. See the
    input and output summary tables of the Element Reference of each
    element that is attached to the node for the available items.

    COORDINATE SYSTEMS: All element nodal results used by ANSOL for
    averaging are in the element coordinate system, except for layered
    elements. Layered element results are in the layer coordinate system.
    You can further specify the element nodal results, for some elements,
    with the SHELL, LAYERP26, and FORCE commands.

    ANSOL does not transform results from RSYS, SOLU to other coordinate
    systems. Verify that all elements attached to the subject node have the
    same coordinate system before using ANSOL.

    SHELL ELEMENTS: The default shell element coordinate system is based on
    node ordering. For shell elements the adjacent elements could have a
    different RSYS,SOLU, making the resultant averaged data inconsistent. A
    note to this effect is issued when ANSOL is used in models containing
    shell elements. Ensure that consistent coordinate systems are active
    for all associated elements used by the ANSOL command.

    DERIVED QUANTITIES: Some of the result items supported by ANSOL (see
    Table: 126:: ANSOL - General Item and Component Labels) are derived
    from the component quantities. Use AVPRIN to specify the principal and
    vector sum quantity averaging methods.

    DEFAULT: If Mat, Real , and Ename are not specified, all of the
    elements attached to the node will be considered. When a material ID,
    real constant ID, or element type discontinuity is detected at a node,
    a note is issued. For example, in a FSI analysis, a FLUID30 element at
    the structure interface would be considered. But since it contains no
    SX result, it will not be used during STORE operations.

    Table: 126:: : ANSOL - General Item and Component Labels

    For more information on the meaning of contact status and its possible
    values, see Reviewing Results in POST1 in the Contact Technology Guide.
    """
    command = f"ANSOL,{nvar},{node},{item},{comp},{name},{mat},{real},{ename}"
    return self.run(command, **kwargs)


def cisol(self, n="", id_="", node="", cont="", dtype="", **kwargs):
    """Stores fracture parameter information in a variable.

    APDL Command: CISOL

    Parameters
    ----------
    n
        Arbitrary reference number or name assigned to this variable.
        Number must be >1 but </= NUMVAR.

    id\_
        Crack ID number.

    node
        Crack tip node number.

    cont
        Contour number.

    dtype
        Data type to output:

        JINT - J-integral

        IIN1 - Interaction integral 1

        IIN2  - Interaction integral 2

        IIN3 - Interaction integral 3

        K1 - Mode 1 stress-intensity factor

        K2 - Mode 2 stress-intensity factor

        K3 - Mode 3 stress-intensity factor

        G1 - Mode 1 energy release rate

        G2  - Mode 2 energy release rate

        G3  - Mode 3 energy release rate

        GT - Total energy release rate

        MFTX - Total material force X

        MFTY - Total material force Y

        MFTZ - Total material force Z

        CEXT - Crack extension
    """
    command = f"CISOL,{n},{id_},{node},{cont},{dtype}"
    return self.run(command, **kwargs)


def data(self, ir="", lstrt="", lstop="", linc="", name="", kcplx="",
         **kwargs):
    """Reads data records from a file into a variable.

    APDL Command: DATA

    Parameters
    ----------
    ir
        Arbitrary reference number assigned to the resulting variable (2 to
        NV [NUMVAR]).  If this number is the same as for a previously
        defined variable, the previously defined variable will be
        overwritten with this result.

    lstrt
        Start at location LSTRT (defaults to 1).

    lstop
        Stop at location LSTOP (defaults to LSTRT).  Maximum location
        available is determined from data previously stored.

    linc
        Fill every LINC location between LSTRT and LSTOP (defaults to 1).

    name
        Eight character name for identifying the variable on the printout
        and displays.  Embedded blanks are compressed upon output.

    kcplx
        Complex number key:

        0 - Data stored as the real part of the complex number.

        1 - Data stored as the imaginary part of the complex number.

    Notes
    -----
    This command must be followed by a format statement (on the next line)
    and the subsequent data records, and all must be on the same file (that
    may then be read with the /INPUT command).  The format specifies the
    number of fields to be read per record, the field width, and the
    placement of the decimal point (if one is not included in the data
    value).  The read operation follows the available FORTRAN FORMAT
    conventions of the system.  See the system FORTRAN manual for details.
    Any standard FORTRAN real format (such as (4F6.0), (F2.0,2X,F12.0),
    etc.) may be used.  Integer (I), character (A), and list-directed (*)
    descriptors may not be used.  The parentheses must be included in the
    format.  Up to 80 columns per record may be read.  Locations may be
    filled within a range.  Previous data in the range will be overwritten.
    """
    command = f"DATA,{ir},{lstrt},{lstop},{linc},{name},{kcplx}"
    return self.run(command, **kwargs)


def edread(self, nstart="", label="", num="", step1="", step2="",
           **kwargs):
    """Reads explicit dynamics output into variables for time-history

    APDL Command: EDREAD
    postprocessing.

    Parameters
    ----------
    nstart
        Starting reference number assigned to the first variable. Allowed
        range is 2 (the default) to NV [NUMVAR].  (NV defaults to 30 for an
        explicit dynamics analysis.)

    label
        Label identifying the output file to be read. No default.

        GLSTAT - Read data from the GLSTAT file.

        MATSUM - Read data from the MATSUM file.

        SPCFORC - Read data from the SPCFORC file.

        RCFORC - Read data from the RCFORC file.

        SLEOUT - Read data from the SLEOUT file.

        NODOUT - Read data from the NODOUT file.

        RBDOUT - Read data from the RBDOUT file.

    num
        Number identifying the data set to be read in (defaults to 1). If
        Label = GLSTAT, NUM is ignored. If Label = MATSUM or RBDOUT, NUM is
        the PART number [EDPART] for which output is desired. If Label =
        SPCFORC or NODOUT, NUM is the node number for which output is
        desired. If Label  = SLEOUT or RCFORC, NUM is the number of the
        contact entity for which output is desired.

    step1, step2
        Load step range of data to be read in. If STEP1 and STEP2 are
        blank, all load steps are read in.

    Notes
    -----
    EDREAD reads data from the specified ascii output file so that it may
    be used during postprocessing. After EDREAD, you must issue the STORE
    command to store the data in time history variables. Once stored, the
    variables can be viewed as plots of output item versus time.

    The number of variables stored depends on the file specified. The
    following table shows the items in each file and the order in which
    they are stored.  If data items were previously stored in variables
    NSTART to NSTART+15, they will be overwritten. If more variables are
    needed, change NV on the NUMVAR command. (Note that hourglass energy
    will not be available if it was not specified for the model
    [EDENERGY,1].)

    The following items under MATSUM are listed in the MATSUM ASCII file
    (in the Mat no. field) for each part number at time intervals specified
    by the EDHTIME command. Use EDREAD,,MATSUM,NUM to specify the part
    number that corresponds to the mat number in the MATSUM file.

    Resultant contact forces and sliding interface energies are available
    from the RCFORC and SLEOUT files, respectively. The RCFORC file is
    written for surface based contact types that include target and contact
    (master and slave) definitions.  You should ensure that this file
    contains valid force results before issuing EDREAD,,RCFORC. Only the
    resultant contact forces on the master surface are available for time-
    history postprocessing.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"EDREAD,{nstart},{label},{num},{step1},{step2}"
    return self.run(command, **kwargs)


def enersol(self, nvar="", item="", name="", **kwargs):
    """Specifies the total energies to be stored.

    APDL Command: ENERSOL

    Parameters
    ----------
    nvar
        Arbitrary reference number assigned to this variable (2 to NV).
    """
    command = f"ENERSOL,{nvar},{item},{name}"
    return self.run(command, **kwargs)


def esol(self, nvar: MapdlInt = "", elem: MapdlInt = "",
         node: MapdlInt = "", item: str = "", comp: str = "",
         name: str = "", **kwargs) -> Optional[str]:
    """Specify element data to be stored from the results file.

    /POST26 APDL Command: ESOL

    Parameters
    ----------
    nvar
        Arbitrary reference number assigned to this variable (2 to
        NV [NUMVAR]). Overwrites any existing results for this
        variable.

    elem
        Element for which data are to be stored.

    node
        Node number on this element for which data are to be
        stored. If blank, store the average element value (except
        for FMAG values, which are summed instead of averaged).

    item
        Label identifying the item. General item labels are shown
        in Table 134: ESOL - General Item and Component Labels
        below. Some items also require a component label.

    comp
        Component of the item (if required). General component
        labels are shown in Table 134: ESOL - General Item and
        Component Labels below.  If Comp is a sequence number (n),
        the NODE field will be ignored.

    name
        Thirty-two character name for identifying the item on the
        printout and displays.  Defaults to a label formed by
        concatenating the first four characters of the Item and
        Comp labels.

    Examples
    --------
    Switch to the time-history postprocessor

    >>> mapdl.post26()

    Store the stress in the X direction for element 1 at node 1

    >>> nvar = 2
    >>> mapdl.esol(nvar, 1, 1, 'S', 'X')

    Move the value to an array and access it via mapdl.parameters

    >>> mapdl.dim('ARR', 'ARRAY', 1)
    >>> mapdl.vget('ARR', nvar)
    >>> mapdl.parameters['ARR']
    array(-1991.40234375)

    Notes
    -----
    See Table: 134:: ESOL - General Item and Component Labels for
    a list of valid item and component labels for element (except
    line element) results.

    The ESOL command defines element results data to be stored
    from a results file (FILE). Not all items are valid for all
    elements. To see the available items for a given element,
    refer to the input and output summary tables in the
    documentation for that element.

    Two methods of data access are available via the ESOL
    command. You can access some simply by using a generic label
    (component name method), while others require a label and
    number (sequence number method).

    Use the component name method to access general element data
    (that is, element data generally available to most element
    types or groups of element types).

    The sequence number method is required for data that is not
    averaged (such as pressures at nodes and temperatures at
    integration points), or data that is not easily described in a
    generic fashion (such as all derived data for structural line
    elements and contact elements, all derived data for thermal
    line elements, and layer data for layered elements).

    Element results are in the element coordinate system, except
    for layered elements where results are in the layer coordinate
    system.  Element forces and moments are in the nodal
    coordinate system. Results are obtainable for an element at a
    specified node. Further location specifications can be made
    for some elements via the SHELL, LAYERP26, and FORCE commands.

    For more information on the meaning of contact status and its
    possible values, see Reviewing Results in POST1 in the Contact
    Technology Guide.
    """
    command = f"ESOL,{nvar},{elem},{node},{item},{comp},{name}"
    return self.run(command, **kwargs)


def file(self, fname="", ext="", **kwargs):
    """Specifies the data file where results are to be found.

    APDL Command: FILE

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    Specifies the ANSYS data file where the results are to be found for
    postprocessing.
    """
    command = f"FILE,{fname},{ext}"
    return self.run(command, **kwargs)


def gapf(self, nvar="", num="", name="", **kwargs):
    """Defines the gap force data to be stored in a variable.

    APDL Command: GAPF

    Parameters
    ----------
    nvar
        Arbitrary reference number assigned to this variable (2 to NV
        [NUMVAR]).  Overwrites any existing results for this variable.

    num
        Number identifying gap number for which the gap force is to be
        stored.  Issue the GPLIST command to display gap numbers.

    name
        Thirty-two character name for identifying the item on the printout
        and displays (defaults to the name GAPF).

    Notes
    -----
    Defines the gap force data to be stored in a variable. Applicable only
    to the expansion pass of the mode-superposition linear transient
    dynamic (ANTYPE,TRANS) analysis. The data is usually on Fname.RDSP.
    """
    command = f"GAPF,{nvar},{num},{name}"
    return self.run(command, **kwargs)


def gssol(self, nvar="", item="", comp="", name="", **kwargs):
    """Specifies which results to store from the results file when using

    APDL Command: GSSOL
    generalized plane strain.

    Parameters
    ----------
    nvar
        Arbitrary reference number or name assigned to this variable.
        Variable numbers can be 2 to NV (NUMVAR) while the name can be an
        eight byte character string. Overwrites any existing results for
        this variable.

    item
        Label identifying item to be stored.

        LENGTH - Change of fiber length at the ending point.

        ROT - Rotation of the ending plane during deformation.

        F - Reaction force at the ending point in the fiber direction.

        M - Reaction moment applied on the ending plane.

    comp
        Component of the item, if Item = ROT or M.

        X - The rotation angle or reaction moment of the ending plane about X.

        Y - The rotation angle or reaction moment of the ending plane about Y.

    name
        Thirty-two character name identifying the item on the printout and
        display. Defaults to the label formed by concatenating the first
        four characters of the Item and Comp labels.

    Notes
    -----
    This command stores the results (new position of the ending plane after
    deformation) for generalized plane strain. All outputs are in the
    global Cartesian coordinate system. For more information about the
    generalized plane strain feature, see Generalized Plane Strain Option
    of Current-Technology Solid Elements in the Element Reference.
    """
    command = f"GSSOL,{nvar},{item},{comp},{name}"
    return self.run(command, **kwargs)


def jsol(self, nvar="", elem="", item="", comp="", name="", **kwargs):
    """Specifies result items to be stored for the joint element.

    APDL Command: JSOL

    Parameters
    ----------
    nvar
        Arbitrary reference number or name assigned to this variable.
        Variable numbers can be 2 to NV (NUMVAR) while the name can be an
        eight-byte character string. Overwrites any existing results for
        this variable.

    elem
        Element number for which to store results.

    item
        Label identifying the item.  Valid item labels are shown in
        Table 202: JSOL - Valid Item and Component Labels below.

    comp
        Component of the Item (if required).  Valid component labels are
        shown in Table 202: JSOL - Valid Item and Component Labels below.

    name
        Thirty-two character name identifying the item on printouts and
        displays.  Defaults to a label formed by concatenating the first
        four characters of the Item and Comp labels.

    Notes
    -----
    This command is valid for the MPC184 joint elements. The values stored
    are for the free or unconstrained degrees of freedom of a joint
    element. Relative reaction forces and moments are available only if
    stiffness, damping, or friction is associated with the joint element.

    Table: 202:: : JSOL - Valid Item and Component Labels


    """
    command = f"JSOL,{nvar},{elem},{item},{comp},{name}"
    return self.run(command, **kwargs)


def nsol(self, nvar="", node="", item="", comp="", name="", sector="",
         **kwargs):
    """Specifies nodal data to be stored from the results file.

    APDL Command: NSOL

    Parameters
    ----------
    nvar
        Arbitrary reference number or name assigned to this variable.
        Variable numbers can be 2 to NV (NUMVAR) while the name can be an
        eight byte character string. Overwrites any existing results for
        this variable.

    node
        Node for which data are to be stored.

    item
        Label identifying the item.  Valid item labels are shown in the
        table below.  Some items also require a component label.

    comp
        Component of the item (if required).  Valid component labels are
        shown in the table below.

    name
        Thirty-two character name identifying the item on printouts and
        displays.  Defaults to a label formed by concatenating the first
        four characters of the Item and Comp labels.

    sector
        For a full harmonic cyclic symmetry solution, the sector number for
        which the results from NODE are to be stored.

    Notes
    -----
    Stores nodal degree of freedom and solution results in a variable. For
    more information, see Data Interpreted in the Nodal Coordinate System
    in the Modeling and Meshing Guide.

    For SECTOR>1, the result is in the nodal coordinate system of the base
    sector, and it is rotated to the expanded sector’s location. Refer to
    Using the /CYCEXPAND Command in the Cyclic Symmetry Analysis Guide for
    more information.

    Table: 211:: : NSOL - Valid Item and Component Labels

    Table: 212:: : NSOL - Valid Item and Component Labels for ANSYS LS-DYNA
    Nodal Results

     For SHELL131 and SHELL132 elements with KEYOPT(3) = 0 or 1, use the
    labels TBOT, TE2, TE3, . . ., TTOP instead of TEMP.
    """
    command = f"NSOL,{nvar},{node},{item},{comp},{name},{sector}"
    return self.run(command, **kwargs)


def nstore(self, tinc="", **kwargs):
    """Defines which time points are to be stored.

    APDL Command: NSTORE

    Parameters
    ----------
    tinc
        Store data associated with every TINC time (or frequency) point(s),
        within the previously defined range of TMIN to TMAX [TIMERANGE].
        (Defaults to 1)

    Notes
    -----
    Defines which time (or frequency) points within the range are to be
    stored.
    """
    command = f"NSTORE,{tinc}"
    return self.run(command, **kwargs)


def numvar(self, nv="", **kwargs):
    """Specifies the number of variables allowed in POST26.

    APDL Command: NUMVAR

    Parameters
    ----------
    nv
        Allow storage for NV variables.  200 maximum are allowed.  Defaults
        to 10 (except for an explicit dynamics analysis, which defaults to
        30).  TIME (variable 1) should also be included in this number.

    Notes
    -----
    Specifies the number of variables allowed for data read from the
    results file and for data resulting from an operation (if any).  For
    efficiency, NV should not be larger than necessary.  NV cannot be
    changed after data storage begins.
    """
    command = f"NUMVAR,{nv}"
    return self.run(command, **kwargs)


def reset(self, **kwargs):
    """Resets all POST1 or POST26 specifications to initial defaults.

    APDL Command: RESET

    Notes
    -----
    Has the same effect as entering the processor the first time within the
    run.  In POST1, resets all specifications to initial defaults, erases
    all element table items, path table data, fatigue table data, and load
    case pointers.  In POST26, resets all specifications to initial
    defaults, erases all variables defined, and zeroes the data storage
    space.
    """
    command = f"RESET,"
    return self.run(command, **kwargs)


def rforce(self, nvar="", node="", item="", comp="", name="", **kwargs):
    """Specifies the total reaction force data to be stored.

    APDL Command: RFORCE

    Parameters
    ----------
    nvar
        Arbitrary reference number assigned to this variable (2 to NV
        [NUMVAR]). Overwrites any existing results for this variable.

    node
        Node for which data are to be stored.  If NODE = P, graphical
        picking is enabled (valid only in the GUI).

    item
        Label identifying the item.  Valid item labels are shown in the
        table below.  Some items also require a component label.

    comp
        Component of the item (if required).  Valid component labels are
        shown in the table below.

    name
        Thirty-two character name identifying the item on printouts and
        displays. Defaults to an eight character label formed by
        concatenating the first four characters of the Item and Comp
        labels.

    Notes
    -----
    Defines the total reaction force data (static, damping, and inertial
    components) to be stored from single pass (ANTYPE,STATIC or TRANS)
    solutions or from the expansion pass of mode-superposition
    (ANTYPE,HARMIC or TRANS) solutions.

    Table: 228:: : RFORCE - Valid Item and Component Labels

    For SHELL131 and SHELL132 elements with KEYOPT(3) = 0 or 1, use the
    labels HBOT, HE2, HE3, . . ., HTOP instead of HEAT.
    """
    command = f"RFORCE,{nvar},{node},{item},{comp},{name}"
    return self.run(command, **kwargs)


def rgb(self, kywrd="", pred="", pgrn="", pblu="", n1="", n2="", ninc="",
        ncntr="", **kwargs):
    """Specifies the RGB color values for indices and contours.

    APDL Command: /RGB

    Parameters
    ----------
    kywrd
        Determines how RGB modifications will be applied.

        INDEX - Specifies that subsequent color values apply to ANSYS color indices (0-15).

        CNTR - Specifies that subsequent color values apply to contours (1-128).  Applies to
               C-option devices only (i.e. X11C or Win32C).

    pred
        Intensity of the color red, expressed as a percentage.

    pgrn
        Intensity of the color green, expressed as a percentage.

    pblu
        Intensity of the color blue, expressed as a percentage.

    n1
        First index (0-15), or contour (1-128) to which the designated RGB
        values apply.

    n2
        Final index (0-15), or contour (1-128) to which the designated RGB
        values apply.

    ninc
        The step increment between the values N1 and N2 determining which
        contours or indices will be controlled by the specified RGB values.

    ncntr
        The new maximum number of contours (1-128).

    Notes
    -----
    Issuing the /CMAP command (with no filename) will restore the default
    color settings.
    """
    command = f"/RGB,{kywrd},{pred},{pgrn},{pblu},{n1},{n2},{ninc},{ncntr}"
    return self.run(command, **kwargs)


def solu(self, nvar="", item="", comp="", name="", **kwargs):
    """Specifies solution summary data per substep to be stored.

    APDL Command: SOLU

    Parameters
    ----------
    nvar
        Arbitrary reference number assigned to this variable (2 to NV
        [NUMVAR]).

    item
        Label identifying the item.  Valid item labels are shown in the
        table below.  Some items may also require a component label.

    comp
        Component of the item (if required).  Valid component labels are
        shown in the table below.  None are currently required.

    name
        Thirty-two character name identifying the item on printouts and
        displays.  Defaults to an eight character label formed by
        concatenating the first four characters of the Item and Comp
        labels.

    Notes
    -----
    See also the PRITER command of POST1 to display some of these items
    directly.  Valid for a static or full transient analysis. All other
    analyses have zeros for the data. Valid item and component labels for
    solution summary values are:
    """
    command = f"SOLU,{nvar},{item},{comp},{name}"
    return self.run(command, **kwargs)


def store(self, lab="", npts="", **kwargs):
    """Stores data in the database for the defined variables.

    APDL Command: STORE

    Parameters
    ----------
    lab
        Valid labels:

        MERGE - Merge data from results file for the time points in memory with the existing
                data using current specifications (default).

        NEW - Store a new set of data, replacing any previously stored data with current
              result file specifications and deleting any previously-
              calculated (OPER) variables. Variables defined using the
              ANSOL command are also deleted.

        APPEN - Append data from results file to the existing data.

        ALLOC - Allocate (and zero) space for NPTS data points.

        PSD - Create a new set of frequency points for PSD calculations (replacing any
              previously stored data and erasing any previously calculated
              data).

    npts
        The number of time points (or frequency points) for storage (used
        only with Lab = ALLOC or PSD).  The value may be input when using
        POST26 with data supplied from other than a results file.  This
        value is automatically determined from the results file data with
        the NEW, APPEN, and MERGE options.  For the PSD option, NPTS
        determines the resolution of the frequency vector (valid numbers
        are between 1 and 10, defaults to 5).

    Notes
    -----
    This command stores data from the results file in the database for the
    defined variables [NSOL, ESOL, SOLU, JSOL] per specification [FORCE,
    LAYERP26, SHELL].  See the Basic Analysis Guide for more information.

    The STORE,PSD command will create a new frequency vector (variable 1)
    for response PSD calculations [RPSD].  This command should first be
    issued before defining variables [NSOL, ESOL, RFORCE] for which
    response PSD's are to be calculated.
    """
    command = f"STORE,{lab},{npts}"
    return self.run(command, **kwargs)


def timerange(self, tmin="", tmax="", **kwargs):
    """Specifies the time range for which data are to be stored.

    APDL Command: TIMERANGE

    Parameters
    ----------
    tmin
        Minimum time (defaults to first time (or frequency) point on the
        file).

    tmax
        Maximum time (defaults to last time (or frequency) point on the
        file).

    Notes
    -----
    Defines the time (or frequency) range for which data are to be read
    from the file and stored in memory.  Use the NSTORE command to define
    the time increment.

    Use PRTIME or PLTIME to specify the time (frequency) range for cyclic
    mode-superposition harmonic analyses.
    """
    command = f"TIMERANGE,{tmin},{tmax}"
    return self.run(command, **kwargs)


def vardel(self, nvar="", **kwargs):
    """Deletes a variable (GUI).

    APDL Command: VARDEL

    Parameters
    ----------
    nvar
        The reference number of the variable to be deleted.  NVAR is as
        defined by NSOL, ESOL, etc.

    Notes
    -----
    Deletes a POST26 solution results variable.  This is a command
    generated by the Graphical User Interface (GUI).  It will appear in the
    log file (Jobname.LOG) if a POST26 variable is deleted from the
    "Defined Time-History Variables" dialog box.  This command is not
    intended to be typed in directly in an ANSYS session (although it can
    be included in an input file for batch input or for use with the /INPUT
    command).
    """
    command = f"VARDEL,{nvar}"
    return self.run(command, **kwargs)


def varnam(self, ir="", name="", **kwargs):
    """Names (or renames) a variable.

    APDL Command: VARNAM

    Parameters
    ----------
    ir
        Reference number of the variable (2 to NV [NUMVAR]).

    name
        Thirty-two character name for identifying variable on printouts and
        displays.  Embedded blanks are compressed for output.
    """
    command = f"VARNAM,{ir},{name}"
    return self.run(command, **kwargs)
